# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from dtest import *
from dtest.util import *


# Ensure that the alternate setUp/tearDown decorators work
class TestAlternate(DTestCase):
    alternate = None

    def setUp(self):
        assert_is_none(self.alternate)
        self.alternate = False

    def tearDown(self):
        assert_false(self.alternate)

    # Should use the default setUp/tearDown
    def test1(self):
        assert_false(self.alternate)

    # Have to use @istest here to make the decorators available
    @istest
    def test2(self):
        assert_true(self.alternate)

    # Alternate setUp/tearDown for test2
    @test2.setUp
    def alternateSetUp(self):
        assert_is_none(self.alternate)
        self.alternate = True

    @test2.tearDown
    def alternateTearDown(self):
        assert_true(self.alternate)



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from dtest import *
from dtest.policy import ThresholdPolicy
from dtest.strategy import SerialStrategy, UnlimitedParallelStrategy, \
    LimitedParallelStrategy
from dtest.test import DTestFixture
from dtest.util import *


class TestThrowaway(DTestCase):
    def test_fordep(self):
        pass


@skip
def test_skip():
    pass


@failing
def test_failing():
    assert False


@attr(attr1=1, attr2=2)
def test_attr():
    pass


@depends(test_skip, test_attr, TestThrowaway.test_fordep)
def test_depends():
    pass


class DecoratorTestException(Exception):
    pass


@raises(DecoratorTestException)
def test_raises():
    raise DecoratorTestException()


@timed(1)
def test_timed():
    pass


@repeat(2)
def test_repeat():
    pass


@parallel
def test_parallel():
    pass


@parallel(2)
def test_parallel_limited():
    pass


@threshold(50)
def test_threshold():
    pass


class ObjTest(object):
    pass


class ResourceTest(Resource):
    def setUp(self):
        t = ObjTest()
        t.test = True
        return t


@require(test=ResourceTest())
def test_require(test):
    assert_true(test.test)


class TestDecorators(DTestCase):
    @depends(test_timed)
    @classmethod
    def setUpClass(cls):
        pass

    @istest
    def skip(self):
        # Verify that skip is true...
        assert_true(test_skip._dt_dtest.skip)

        # Verify that it's false on something else
        assert_false(test_failing._dt_dtest.skip)

    @istest
    def failing(self):
        # Verify that failing is true...
        assert_true(test_failing._dt_dtest.failing)

        # Verify that it's false on something else
        assert_false(test_skip._dt_dtest.failing)

    @istest
    def attr(self):
        # Verify that the attributes are set as expected
        assert_equal(test_attr._dt_dtest.attr1, 1)
        assert_equal(test_attr._dt_dtest.attr2, 2)

    @istest
    def depends(self):
        # Part 1: Verify that test_depends() is dependent on
        # test_skip() and test_attr()
        assert_in(test_skip._dt_dtest, test_depends._dt_dtest.dependencies)
        assert_in(test_attr._dt_dtest, test_depends._dt_dtest.dependencies)
        assert_in(TestThrowaway.test_fordep._dt_dtest,
                  test_depends._dt_dtest.dependencies)

        # Part 2: Verify that test_depends() is in the depedents set
        # of test_skip() and test_attr()
        assert_in(test_depends._dt_dtest, test_skip._dt_dtest.dependents)
        assert_in(test_depends._dt_dtest, test_attr._dt_dtest.dependents)
        assert_in(test_depends._dt_dtest,
                  TestThrowaway.test_fordep._dt_dtest.dependents)

    @istest
    def raises(self):
        # Verify that the set of expected exceptions is as expected
        assert_set_equal(test_raises._dt_dtest.raises,
                         set([DecoratorTestException]))

        # Verify that it's the empty set on something else
        assert_set_equal(test_timed._dt_dtest.raises, set())

    @istest
    def timed(self):
        # Verify that the timeout is set properly
        assert_equal(test_timed._dt_dtest.timeout, 1)

        # Verify that it's None on something else
        assert_is_none(test_raises._dt_dtest.timeout)

    @istest
    def repeat(self):
        # Verify that the repeat count is set properly
        assert_equal(test_repeat._dt_dtest.repeat, 2)

        # Verify that it's 1 on something else
        assert_equal(test_timed._dt_dtest.repeat, 1)

    @istest
    def parallel(self):
        # Verify that the strategy is set properly
        assert_is_instance(test_parallel._dt_dtest._strategy,
                           UnlimitedParallelStrategy)

        # Verify that it's SerialStrategy on something else
        assert_is_instance(test_timed._dt_dtest._strategy, SerialStrategy)

    @istest
    def parallel_limited(self):
        # Verify that the strategy is set properly
        assert_is_instance(test_parallel_limited._dt_dtest._strategy,
                           LimitedParallelStrategy)

        # Verify that the limit is set properly
        assert_equal(test_parallel_limited._dt_dtest._strategy.limit, 2)

    @istest
    def threshold(self):
        # Verify that the policy is set properly
        assert_is_instance(test_threshold._dt_dtest._policy, ThresholdPolicy)

        # Verify that the threshold is set properly
        assert_almost_equal(test_threshold._dt_dtest._policy.threshold, 50.0)

    @istest
    def isfixture(self):
        # Verify that setUpClass has a fixture associated with it
        assert_is_instance(self.setUpClass._dt_dtest, DTestFixture)

        # Verify that we have the appropriate dependencies
        assert_in(test_timed._dt_dtest, self.setUpClass._dt_dtest.dependencies)

        # Verify that we have the appropriate dependents
        assert_in(self.setUpClass._dt_dtest, test_timed._dt_dtest.dependents)

    @istest
    def require(self):
        # Verify that test_require has a resource associated with it
        assert_is_instance(test_require._dt_dtest._resources, dict)
        assert_equal(len(test_require._dt_dtest._resources), 1)
        assert_in('test', test_require._dt_dtest._resources)
        assert_is_instance(test_require._dt_dtest._resources['test'],
                           ResourceTest)

        # Verify that everything else has empty resources
        assert_equal(test_threshold._dt_dtest._resources, {})



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
==============
Test Utilities
==============

This module contains a number of utilities for use by tests.  The use
of these utilities is not mandatory--a simple ``assert`` statement
will work fine--but the utilities provided may assist in evaluating
complicated assertions.  Most of these utilities are similar to the
unittest.TestCase methods of similar names--for instance, the
assert_false() utility is identical in action to
unittest.TestCase.assertFalse().
"""

import re

from dtest.exceptions import DTestException


__all__ = ['assert_false', 'assert_true', 'assert_raises', 'assert_equal',
           'assert_not_equal', 'assert_almost_equal',
           'assert_not_almost_equal', 'assert_sequence_equal',
           'assert_list_equal', 'assert_tuple_equal', 'assert_set_equal',
           'assert_in', 'assert_not_in', 'assert_is', 'assert_is_not',
           'assert_dict_equal', 'assert_dict_contains', 'assert_items_equal',
           'assert_less', 'assert_less_equal', 'assert_greater',
           'assert_greater_equal', 'assert_is_none', 'assert_is_not_none',
           'assert_is_instance', 'assert_is_not_instance',
           'assert_regexp_matches', 'assert_not_regexp_matches']


def safe_repr(obj, maxlen=None):
    """
    Helper function to safely determine the representation of ``obj``.
    This function can be used in the case that the user-provided
    __repr__() method raises an exception.  The ``maxlen`` argument,
    if given, provides a maximum length for the representation.
    """

    # Safely get the representation of an object
    try:
        result = repr(obj)
    except:
        # The repr() could call user code, so if it fails, we want to
        # be intelligent about what we return
        result = object.__repr__(obj)

    # Truncate representation if necessary
    if maxlen is not None and len(result) > maxlen:
        result = result[:maxlen - 3] + '...'

    return result


def select_msg(usermsg, defmsg):
    """
    Helper function to select a message.  If ``usermsg`` is None,
    ``defmsg`` will be returned; otherwise, ``usermsg`` will be
    returned.  This allows users to specify alternate messages to emit
    if an assertion fails.
    """

    # Select the correct message to use
    if usermsg is None:
        return defmsg
    return usermsg


def make_re(regexp):
    """
    Helper function to build a regular expression object.  If
    ``regexp`` is a string, it will be compiled into a regular
    expression object; otherwise, ``regexp`` will be returned
    unmodified.
    """

    # If it's None or not an instance of string, return it
    if regexp is None or not isinstance(regexp, basestring):
        return regexp

    # Convert to a regular expression
    return re.compile(regexp)


def assert_false(expr, msg=None):
    """
    Assert that ``expr`` evaluate to False.
    """

    # Ensure expr is False
    if expr:
        msg = select_msg(msg, "%s is not False" % safe_repr(expr))
        raise AssertionError(msg)


def assert_true(expr, msg=None):
    """
    Assert that ``expr`` evaluate to True.
    """

    # Ensure expr is True
    if not expr:
        msg = select_msg(msg, "%s is not True" % safe_repr(expr))
        raise AssertionError(msg)


class AssertRaisesContext(object):
    """
    AssertRaisesContext
    ===================

    The AssertRaisesContext class is used by the assert_raises()
    function as a context manager.  It ensures that the statement
    executed within the context raises the expected exception(s).
    """

    def __init__(self, excs, msg, regexp=None):
        """
        Initializes an AssertRaisesContext object.  The ``excs``
        argument should be a tuple of legal exceptions (None indicates
        that not raising an exception is also legal), and ``msg`` is
        the user-specified message.  If ``regexp`` is not None, then
        the exception raised must match the regular expression.
        """

        self.excs = excs
        self.msg = msg
        self.regexp = make_re(regexp)

    def __enter__(self):
        """
        Enters the context manager.  Returns the context itself.
        """

        return self

    def __exit__(self, exc_type, exc_value, tb):
        """
        Exits the context manager.  Compares any exception raised to
        the expectations set when this context manager was
        initialized.  If the expectations are not met, an
        AssertionError will be raised.
        """

        # Ensure the appropriate exception was raised
        if exc_type is None:
            if None in self.excs:
                # Exception wasn't raised, but that's OK
                return True

            # OK, have to raise an assertion error
            msg = select_msg(self.msg, "No exception raised; expected one "
                             "of: (%s)" % ', '.join([self._exc_name(exc)
                                                     for exc in self.excs]))
            raise AssertionError(msg)

        # OK, an exception was raised; make sure it's one we were
        # expecting
        elif exc_type in self.excs or issubclass(exc_type, self.excs):
            # Do we need to check against a regexp?
            if (self.regexp is not None and
                not self.regexp.search(str(exc_value))):
                msg = select_msg(self.msg, 'Exception "%s" does not match '
                                 'expression "%s"' % (exc_value,
                                                      self.regexp.pattern))
                raise AssertionError(msg)

            # Assertion we were looking for, so say we handled it
            return True

        # Not an exception we were expecting, so let it bubble up
        return False

    @staticmethod
    def _exc_name(exc):
        """
        Helper method to safely determine an exception's name.
        """

        try:
            # If it has a name, return it
            return exc.__name__
        except AttributeError:
            # OK, let's try to stringify it instead
            return str(exc)


def assert_raises(excepts, *args, **kwargs):
    """
    Assert that an exception specified by ``excepts`` is raised.  If
    ``excepts`` is a tuple, any listed exception will be acceptable;
    if None is specified as an exception, not raising an exception
    will also be acceptable.  With no other arguments, returns a
    context manager that can be used in a ``with`` statement.  If one
    additional argument is specified, or the ``callableObj`` keyword
    argument is specified, the callable so specified will be called
    directly.  A ``callableObj`` keyword argument overrides any
    callables specified as additional arguments.  All remaining
    arguments and keyword arguments will be passed to the callable.

    There are two other special-purpose keyword arguments.  The
    ``noRaiseMsg`` keyword argument may be used to specify an
    alternate message to use in the event that no exception is raised
    by the callable.  The ``matchRegExp`` keyword argument may be used
    to specify that the exception must match the given regular
    expression, which must be either a string or a regular expression
    object supporting the search() method.  If present, these keyword
    arguments will be removed from the set of keyword arguments before
    calling the callable.
    """

    # Extract callableObj from arguments
    callableObj = None
    if 'callableObj' in kwargs:
        callableObj = kwargs['callableObj']
        del kwargs['callableObj']
    elif args:
        callableObj = args[0]
        args = args[1:]

    # Extract noRaiseMsg from keyword arguments
    noRaiseMsg = None
    if 'noRaiseMsg' in kwargs:
        noRaiseMsg = kwargs['noRaiseMsg']
        del kwargs['noRaiseMsg']

    # Extract matchRegExp from keyword arguments
    matchRegExp = None
    if 'matchRegExp' in kwargs:
        matchRegExp = kwargs['matchRegExp']
        del kwargs['matchRegExp']

    # First, check if excepts is a sequence
    try:
        length = len(excepts)
        excepts = tuple(excepts)
    except (TypeError, NotImplementedError):
        excepts = (excepts,)

    # Now, grab a context
    ctx = AssertRaisesContext(excepts, noRaiseMsg, matchRegExp)

    if callableObj is None:
        # No callable, so just return the context
        return ctx

    # Execute the callable
    with ctx:
        callableObj(*args, **kwargs)


def assert_equal(first, second, msg=None):
    """
    Assert that ``first`` is equal to ``second``.
    """

    # Ensure first == second
    if not first == second:
        msg = select_msg(msg, "%s != %s" %
                         (safe_repr(first), safe_repr(second)))
        raise AssertionError(msg)


def assert_not_equal(first, second, msg=None):
    """
    Assert that ``first`` is not equal to ``second``.
    """

    # Ensure first != second
    if not first != second:
        msg = select_msg(msg, "%s == %s" %
                         (safe_repr(first), safe_repr(second)))
        raise AssertionError(msg)


def assert_almost_equal(first, second, msg=None, places=None, delta=None):
    """
    Assert that ``first`` and ``second`` are almost equal.  Comparison
    can be done either to within a given number of decimal places (7,
    by default), or to within a given delta.  The delta is given using
    the ``delta`` keyword argument, or the number of places can be
    specified using ``places``.  Note that ``places`` and ``delta``
    cannot both be specified.
    """

    # Sanity-check arguments
    if places is not None and delta is not None:
        raise DTestException("Specify delta or places, not both")

    # Ensure first and second are similar
    if first == second:
        # Short-circuit for the simple case
        return

    # Is this comparison a delta style?
    if delta is not None:
        if abs(first - second) <= delta:
            return

        stdmsg = "%s != %s within %s delta" % (safe_repr(first),
                                               safe_repr(second),
                                               safe_repr(delta))

    else:
        # OK, do a places-based comparison; default places to 7
        if places is None:
            places = 7

        if round(abs(first - second), places) == 0:
            return

        stdmsg = "%s != %s within %s places" % (safe_repr(first),
                                                safe_repr(second),
                                                safe_repr(places))

    # OK, they're not equal, so tell the caller
    raise AssertionError(select_msg(msg, stdmsg))


def assert_not_almost_equal(first, second, msg=None, places=None, delta=None):
    """
    Assert that ``first`` and ``second`` are not almost equal.
    Comparison can be done either to within a given number of decimal
    places (7, by default), or to within a given delta.  The delta is
    given using the ``delta`` keyword argument, or the number of
    places can be specified using ``places``.  Note that ``places``
    and ``delta`` cannot both be specified.
    """

    # Sanity-check arguments
    if places is not None and delta is not None:
        raise DTestException("Specify delta or places, not both")

    # Is this comparison a delta style?
    if delta is not None:
        if not (first == second) and abs(first - second) > delta:
            return

        stdmsg = "%s == %s within %s delta" % (safe_repr(first),
                                               safe_repr(second),
                                               safe_repr(delta))

    else:
        # OK, do a places-based comparison; default places to 7
        if places is None:
            places = 7

        if not (first == second) and round(abs(first - second), places) != 0:
            return

        stdmsg = "%s == %s within %s places" % (safe_repr(first),
                                                safe_repr(second),
                                                safe_repr(places))

    # OK, they're not equal, so tell the caller
    raise AssertionError(select_msg(msg, stdmsg))


def assert_sequence_equal(seq1, seq2, msg=None, seq_type=None):
    """
    Assert that ``seq1`` and ``seq2`` have the same contents.  If
    ``seq_type`` is specified, both sequences must be of that type.
    """

    # Enforce sequence typing
    if seq_type is not None:
        st_name = seq_type.__name__
        if not isinstance(seq1, seq_type):
            raise AssertionError("First sequence is not a %s: %s" %
                                 (st_name, safe_repr(seq1)))
        if not isinstance(seq2, seq_type):
            raise AssertionError("Second sequence is not a %s: %s" %
                                 (st_name, safe_repr(seq2)))
    else:
        st_name = "sequence"

    # Grab the lengths of the sequences
    differing = None
    try:
        len1 = len(seq1)
    except (TypeError, NotImplementedError):
        differing = "First %s has no length.  Non-sequence?" % st_name
    if differing is None:
        try:
            len2 = len(seq2)
        except (TypeError, NotImplementedError):
            differing = "Second %s has no length.  Non-sequence?" % st_name

    # Now let's compare the sequences
    if differing is None:
        if seq1 == seq2:
            return

        # They differ somehow...
        seq1_repr = safe_repr(seq1)
        seq2_repr = safe_repr(seq2)
        if len(seq1_repr) > 30:
            seq1_repr = seq1_repr[:27] + "..."
        if len(seq2_repr) > 30:
            seq2_repr = seq2_repr[:27] + "..."
        differing = "%ss differ: %s != %s" % (st_name.capitalize(),
                                              seq1_repr, seq2_repr)

        # Compare sequences element by element
        for i in xrange(min(len1, len2)):
            try:
                item1 = seq1[i]
            except (TypeError, IndexError, NotImplementedError):
                differing += ("\nUnable to index element %d of first %s" %
                              (i, st_name))
                break

            try:
                item2 = seq2[i]
            except (TypeError, IndexError, NotImplementedError):
                differing += ("\nUnable to index element %d of second %s" %
                              (i, st_name))
                break

            if item1 != item2:
                differing += ("\nFirst differing element %d: %s != %s" %
                              (i, safe_repr(item1), safe_repr(item2)))
                break
        else:
            # The items tally up, but...
            if len1 == len2 and seq_type is None and type(seq1) != type(seq2):
                # Just differ in type; who cares?
                return

        # Emit the extra elements
        if len1 > len2:
            differing += ("\nFirst %s contains %d additional elements" %
                          (st_name, len1 - len2))
            try:
                differing += ("\nFirst extra element %d: %s" %
                              (len2, safe_repr(seq1[len2])))
            except (TypeError, IndexError, NotImplementedError):
                differing += ("\nUnable to index element %d of first %s" %
                              (len2, st_name))
        elif len1 < len2:
            differing += ("\nSecond %s contains %d additional elements" %
                          (st_name, len2 - len1))
            try:
                differing += ("\nFirst extra element %d: %s" %
                              (len1, safe_repr(seq2[len1])))
            except (TypeError, IndexError, NotImplementedError):
                differing += ("\nUnable to index element %d of second %s" %
                              (len1, st_name))

    # Not going to bother with the whole diff stuff unittest does
    raise AssertionError(select_msg(msg, differing))


def assert_list_equal(list1, list2, msg=None):
    """
    Assert that lists ``list1`` and ``list2`` have the same contents.
    """

    assert_sequence_equal(list1, list2, msg=msg, seq_type=list)


def assert_tuple_equal(tuple1, tuple2, msg=None):
    """
    Assert that tuples ``tuple1`` and ``tuple2`` have the same
    contents.
    """

    assert_sequence_equal(tuple1, tuple2, msg=msg, seq_type=tuple)


def assert_set_equal(set1, set2, msg=None):
    """
    Assert that sets ``set1`` and ``set2`` have the same contents.
    """

    # Obtain the two set differences
    try:
        diff1 = set1.difference(set2)
    except TypeError as e:
        raise AssertionError("Invalid type when attempting set "
                             "difference: %s" % e)
    except AttributeError as e:
        raise AssertionError("First set does not support set "
                             "difference: %s" % e)

    try:
        diff2 = set2.difference(set1)
    except TypeError as e:
        raise AssertionError("Invalid type when attempting set "
                             "difference: %s" % e)
    except AttributeError as e:
        raise AssertionError("Second set does not support set "
                             "difference: %s" % e)

    # If both differences are empty, then we're fine
    if not (diff1 or diff2):
        return

    # Accumulate items in one but not the other
    stdmsg = ''
    if diff1:
        stdmsg += ("Items in the first set but not the second: %s" %
                   ', '.join([safe_repr(item) for item in diff1]))
    if diff2:
        if stdmsg:
            stdmsg += "\n"
        stdmsg += ("Items in the second set but not the first: %s" %
                   ', '.join([safe_repr(item) for item in diff2]))

    # Tell the caller
    raise AssertionError(select_msg(msg, stdmsg))


def assert_in(member, container, msg=None):
    """
    Assert that ``member`` is in ``container``.
    """

    # Ensure member is in container
    if member not in container:
        msg = select_msg(msg, "%s not found in %s" %
                         (safe_repr(member), safe_repr(container)))
        raise AssertionError(msg)


def assert_not_in(member, container, msg=None):
    """
    Assert that ``member`` is not in ``container``.
    """

    # Ensure member is not in container
    if member in container:
        msg = select_msg(msg, "%s unexpectedly found in %s" %
                         (safe_repr(member), safe_repr(container)))
        raise AssertionError(msg)


def assert_is(expr1, expr2, msg=None):
    """
    Assert that ``expr1`` is ``expr2``.
    """

    # Ensure expr1 is expr2
    if expr1 is not expr2:
        msg = select_msg(msg, "%s is not %s" %
                         (safe_repr(expr1), safe_repr(expr2)))
        raise AssertionError(msg)


def assert_is_not(expr1, expr2, msg=None):
    """
    Assert that ``expr1`` is not ``expr2``.
    """

    # Ensure expr1 is not expr2
    if expr1 is expr2:
        msg = select_msg(msg, "%s is unexpectedly %s" %
                         (safe_repr(expr1), safe_repr(expr2)))
        raise AssertionError(msg)


def assert_dict_equal(d1, d2, msg=None):
    """
    Assert that dictionaries ``d1`` and ``d2`` have the same contents.
    """

    # Make sure both are dict instances
    if not isinstance(d1, dict):
        raise AssertionError("First argument is not a dictionary")
    if not isinstance(d2, dict):
        raise AssertionError("Second argument is not a dictionary")

    # Ensure they're equal
    if d1 != d2:
        stdmsg = "%s != %s" % (safe_repr(d1, 30), safe_repr(d2, 30))
        raise AssertionError(select_msg(msg, stdmsg))


def assert_dict_contains(actual, expected, msg=None):
    """
    Assert that the dictionary ``actual`` contains the elements in
    ``expected``; extra elements are ignored.
    """

    # Determine missing or mismatched keys
    missing = []
    mismatched = []
    for k, v in expected.items():
        if k not in actual:
            missing.append(k)
        elif v != actual[k]:
            mismatched.append("Key %s: expected %s, actual %s" %
                              (safe_repr(k), safe_repr(v),
                               safe_repr(actual[k])))

    # Are there any problems?
    if not (missing or mismatched):
        return

    # Build up the standard message
    stdmsg = ''
    if missing:
        stdmsg += "Missing keys: %s" % ', '.join([safe_repr(k)
                                                  for k in missing])
    if mismatched:
        if stdmsg:
            stdmsg += '; '
        stdmsg += "Mismatched values: %s" % '; '.join(mismatched)

    raise AssertionError(select_msg(msg, stdmsg))


def assert_items_equal(actual, expected, msg=None):
    """
    Assert that ``actual`` and ``expected`` contain the same items.
    Note that this function is implemented using an n^2 algorithm and
    so should likely not be used if ``actual`` or ``expected`` contain
    large numbers of items.
    """

    # Order n^2 algorithm for comparing items in the lists
    missing = []
    while expected:
        item = expected.pop()
        try:
            # Take it out of what we actually got
            actual.remove(item)
        except ValueError:
            # It wasn't there!
            missing.append(item)

    # Now, missing contains those items in expected which were not in
    # actual, and actual contains those items which were not in
    # expected; if missing and actual are empty, we're fine
    if not missing and not actual:
        return

    # Build the error message
    stdmsg = ''
    if missing:
        stdmsg += ("Missing items: %s" %
                   ', '.join([safe_repr(i) for i in missing]))
    if actual:
        if stdmsg:
            stdmsg += '; '
        stdmsg += ("Unexpected items: %s" %
                   ', '.join([safe_repr(i) for i in actual]))
    raise AssertionError(select_msg(msg, stdmsg))


def assert_less(a, b, msg=None):
    """
    Assert that ``a`` is less than ``b``.
    """

    # Ensure a < b
    if not a < b:
        msg = select_msg(msg, "%s not less than %s" %
                         (safe_repr(a), safe_repr(b)))
        raise AssertionError(msg)


def assert_less_equal(a, b, msg=None):
    """
    Assert that ``a`` is less than or equal to ``b``.
    """

    # Ensure a <= b
    if not a <= b:
        msg = select_msg(msg, "%s not less than or equal to %s" %
                         (safe_repr(a), safe_repr(b)))
        raise AssertionError(msg)


def assert_greater(a, b, msg=None):
    """
    Assert that ``a`` is greater than ``b``.
    """

    # Ensure a > b
    if not a > b:
        msg = select_msg(msg, "%s not greater than %s" %
                         (safe_repr(a), safe_repr(b)))
        raise AssertionError(msg)


def assert_greater_equal(a, b, msg=None):
    """
    Assert that ``a`` is greater than or equal to ``b``.
    """

    # Ensure a >= b
    if not a >= b:
        msg = select_msg(msg, "%s not greater than or equal to %s" %
                         (safe_repr(a), safe_repr(b)))
        raise AssertionError(msg)


def assert_is_none(obj, msg=None):
    """
    Assert that ``obj`` is None.
    """

    # Ensure obj is None
    if obj is not None:
        msg = select_msg(msg, "%s is not None" % safe_repr(obj))
        raise AssertionError(msg)


def assert_is_not_none(obj, msg=None):
    """
    Assert that ``obj`` is not None.
    """

    # Ensure obj is not None
    if obj is None:
        msg = select_msg(msg, "%s is None" % safe_repr(obj))
        raise AssertionError(msg)


def assert_is_instance(obj, cls, msg=None):
    """
    Assert that ``obj`` is an instance of class ``cls``.
    """

    # Ensure obj is an instance of cls
    if not isinstance(obj, cls):
        msg = select_msg(msg, "%s is not an instance of %r" %
                         (safe_repr(obj), cls))
        raise AssertionError(msg)


def assert_is_not_instance(obj, cls, msg=None):
    """
    Assert that ``obj`` is not an instance of class ``cls``.
    """

    # Ensure obj is not an instance of cls
    if isinstance(obj, cls):
        msg = select_msg(msg, "%s is an instance of %r" %
                         (safe_repr(obj), cls))
        raise AssertionError(msg)


def assert_regexp_matches(text, regexp, msg=None):
    """
    Assert that ``text`` matches the regular expression ``regexp``.
    The regular expression may be either a string or a regular
    expression object supporting the search() method.
    """

    # Get the regular expression
    regexp = make_re(regexp)

    # Does it match?
    if not regexp.search(text):
        msg = select_msg(msg, "'%s' does not match text %s" %
                         (regexp.pattern, safe_repr(text)))
        raise AssertionError(msg)


def assert_not_regexp_matches(text, regexp, msg=None):
    """
    Assert that ``text`` does not match the regular expression
    ``regexp``.  The regular expression may be either a string or a
    regular expression object supporting the search() method.
    """

    # Get the regular expression
    regexp = make_re(regexp)

    # Does it match?
    match = regexp.search(text)
    if match:
        msg = select_msg(msg, "'%s' matches text %r from %s" %
                         (regexp.pattern, text[match.start():match.end()],
                          safe_repr(text)))
        raise AssertionError(msg)



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

t_order = []


def setUp():
    # Make sure we're the first thing to have run
    assert len(t_order) == 0, "Ordering error running test suite"

    # Keep track of what has run
    t_order.append('tests.setUp')


def tearDown():
    # Make sure we're the fourteenth thing to have run
    assert len(t_order) == 13, "Ordering error running test suite"
    assert t_order[-1] == 'tests.ordering.tearDown', "Incorrect previous step"

    # Keep track of what has run
    t_order.append('tests.tearDown')



#!/usr/bin/python
#
# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

import dtest
from dtest import util


exp_order = [
    'tests.setUp',
    'tests.ordering.setUp',
    'tests.ordering.test_order.setUp',
    'tests.ordering.test_order.OrderingTestCase.setUpClass',
    'tests.ordering.test_order.OrderingTestCase.setUp',
    'tests.ordering.test_order.OrderingTestCase.test1',
    'tests.ordering.test_order.OrderingTestCase.tearDown',
    'tests.ordering.test_order.OrderingTestCase.setUp',
    'tests.ordering.test_order.OrderingTestCase.test2',
    'tests.ordering.test_order.OrderingTestCase.tearDown',
    'tests.ordering.test_order.OrderingTestCase.tearDownClass',
    'tests.ordering.test_order.tearDown',
    'tests.ordering.tearDown',
    'tests.tearDown',
    ]

required = [
    'tests.explore.a_test',
    'tests.explore.test_discovered',
    'tests.explore.test_pkg.a_test',
    'tests.explore.test_pkg.test_discovered',
    'tests.explore.pkg_impl.a_test',
    'tests.explore.pkg_impl.test_discovered',
    'tests.explore.pkg_impl.test_impl.a_test',
    'tests.explore.pkg_impl.test_impl.test_discovered',
    ]

prohibited = [
    'tests.explore.test_not',
    'tests.explore.pkg.a_test',
    'tests.explore.pkg.test_not',
    'tests.explore.pkg.test_discovered',
    'tests.explore.pkg.nottest.a_test',
    'tests.explore.pkg.nottest.test_not',
    'tests.explore.pkg.nottest.test_discovered',
    'tests.explore.notpkg.a_test',
    'tests.explore.notpkg.test_not',
    'tests.explore.notpkg.test_discovered',
    'tests.explore.test_pkg.test_not',
    'tests.explore.test_notpkg.a_test',
    'tests.explore.test_notpkg.test_not',
    'tests.explore.test_notpkg.test_discovered',
    'tests.explore.pkg_impl.test_not',
    'tests.explore.pkg_impl.test_impl.test_not',
    ]


@dtest.istest
def test_ordering():
    # Look up t_order and make sure it's right
    t_order = sys.modules['tests'].t_order
    util.assert_list_equal(t_order, exp_order)


@dtest.istest
def test_discovery():
    # Get the list of test names
    tnames = set([str(t) for t in queue.tests])

    # Now go through the required list and make sure all those tests
    # are present
    for t in required:
        assert t in tnames, "Required test %r not discovered" % t

    # And similarly for the prohibited list
    for t in prohibited:
        assert t not in tnames, "Prohibited test %r discovered" % t


@dtest.istest
def test_partner_setUp():
    # Look up setUpRun and make sure it's right
    setUpRun = sys.modules['tests.test_partner'].setUpRun
    util.assert_false(setUpRun)


@dtest.istest
def test_partner_tearDown():
    # Look up tearDownRun and make sure it's right
    tearDownRun = sys.modules['tests.test_partner'].tearDownRun
    util.assert_false(tearDownRun)


# Start by processing the command-line arguments
(options, args) = dtest.optparser(usage="%prog [options]").parse_args()

# Get the options
opts = dtest.opts_to_args(options)

# If directory isn't set, use "tests"
if 'directory' not in opts:
    opts['directory'] = 'tests'

# Need to allocate a queue; select some suboptions for the task
subopts = {'skip': lambda dt: hasattr(dt, 'must_skip') and dt.must_skip}
if 'maxth' in opts:
    subopts['maxth'] = opts['maxth']
if 'output' in opts:
    subopts['output'] = opts['output']
queue = dtest.DTestQueue(**subopts)

# OK, we need to do the explore
dtest.explore(opts['directory'], queue)

# Now, set up the dependency between tests.tearDown and our
# test_ordering() test and the test_partner_*() tests
dtest.depends(sys.modules['tests'].tearDown)(test_ordering)
dtest.depends(sys.modules['tests'].tearDown)(test_partner_setUp)
dtest.depends(sys.modules['tests'].tearDown)(test_partner_tearDown)

# Have to add local tests to tests set
queue.add_test(test_ordering)
queue.add_test(test_discovery)
queue.add_test(test_partner_setUp)
queue.add_test(test_partner_tearDown)

# Implement the rest of dtest.main()
if not opts.get('dryrun', False):
    # Execute the tests
    result = queue.run(opts.get('debug', False))
else:
    result = True

    # Print out the names of the tests
    print "Discovered tests:\n"
    for dt in queue.tests:
        if dt.istest():
            print str(dt)

# Are we to dump the dependency graph?
if 'dotpath' in opts:
    with open(opts['dotpath'], 'w') as f:
        print >>f, queue.dot()

# All done!
sys.exit(not result)



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
==============
Test Resources
==============

This module contains the Resource class, along with a number of other
functions, which together provide the functionality for providing test
resources.  A test resource is simply a discrete object which is
required by a given test function or method; it could be a temporary
file containing configuration for a component, or a client object for
accessing a server, or virtually anything that would ordinarily be set
up in a test fixture.  Resources which are not modified (which are not
"dirty") may be reused by following tests, subject to threading
constraints.

This file does not contain the @require() decorator, which is defined
in the dtest.test module.
"""

import functools
import sys

from eventlet import semaphore


class ResourceObjectMeta(type):
    """
    ResourceObjectMeta class, which provides some utility class
    methods for the ResourceObject class without polluting its
    namespace.
    """

    def resource(cls, obj):
        """
        Retrieve the resource associated with a ResourceObject
        ``obj``.
        """

        return obj.__res_resource__

    def obj(cls, obj):
        """
        Retrieve the actual object associated with a ResourceObject
        ``obj``.
        """

        return obj.__res_obj__

    def dirty(cls, obj, new=None):
        """
        Retrieve the dirty flag associated with a ResourceObject
        ``obj``.  If ``new`` is provided, specifies a new value for
        the dirty flag.
        """

        # Save the current value
        old = obj.__res_dirty__

        # Update it
        if new is not None:
            obj.__res_dirty__ = new

        return old

    def makedirty(cls, obj, new):
        """
        Changes the 'makedirty' flag associated with a ResourceObject
        ``obj``.  Returns its previous value.
        """

        # Save the current value
        old = obj.__res_makedirty__

        # Update it
        obj.__res_makedirty__ = new

        return old


class ResourceObject(object):
    """
    ResourceObject class, which acts as a binding between a Resource
    and the actual resource object.  Acts as a transparent proxy, but
    calling any of the dirty methods or setting or deleting any
    attributes causes the resource to be marked as dirty.  Use the
    cleanaccess() function as a context manager to allow accesses that
    would normally dirty the resource.
    """

    __metaclass__ = ResourceObjectMeta

    def __init__(self, resource, resobj, dirtymeths):
        """
        Create a ResourceObject.  The ``resource`` is an instance of
        Resource, and ``resobj`` is the actual object to delegate
        accesses to.  The ``dirtymeths`` is a list of methods which,
        when called, cause the object to become dirty.
        """

        # We use __res_*__ to avoid conflicting with attributes on the
        # resource object
        self.__res_resource__ = resource
        self.__res_obj__ = resobj
        self.__res_dirtymeths__ = set(dirtymeths)
        self.__res_dirty__ = False
        self.__res_makedirty__ = True

    def __getattribute__(self, name):
        """
        Retrieves an attribute from the resource object.  If the
        attribute is a dirty method, it will be wrapped with a
        function which will set the dirty attribute on the resource.
        """

        # Short-cut the __res_*__ attributes...
        if name.startswith('__res_') and name.endswith('__'):
            return super(ResourceObject, self).__getattribute__(name)

        # First, get the attribute from the resource object
        attr = getattr(self.__res_obj__, name)

        # If it's a callable, and listed in dirtymeths, wrap it
        if callable(attr) and name in self.__res_dirtymeths__:
            @functools.wraps(attr)
            def wrapper(*args, **kwargs):
                # Mark it dirty
                if self.__res_makedirty__:
                    self.__res_dirty__ = True

                # Call the function
                return attr(*args, **kwargs)

            # Return our wrapper
            return wrapper

        return attr

    def __setattr__(self, name, value):
        """
        Sets an attribute on the resource object.  Sets the dirty
        attribute on the resource.
        """

        # Short-cut the __res_*__ attributes...
        if name.startswith('__res_') and name.endswith('__'):
            return super(ResourceObject, self).__setattr__(name, value)

        # OK, mark resource dirty
        if self.__res_makedirty__:
            self.__res_dirty__ = True

        # Delegate to the resource object
        return setattr(self.__res_obj__, name, value)

    def __delattr__(self, name):
        """
        Deletes an attribute on the resource object.  Sets the dirty
        attribute on the resource.
        """

        # Short-cut the __res_*__ attributes...
        if name.startswith('__res_') and name.endswith('__'):
            return super(ResourceObject, self).__delattr__(name)

        # OK, mark resource dirty
        if self.__res_makedirty__:
            self.__res_dirty__ = True

        # Delegate to the resource object
        return delattr(self.__res_obj__, name)


class CleanContext(object):
    """
    Context manager returned by cleanaccess().  Temporarily disables
    dirty detection for resource objects.
    """

    def __init__(self, objs):
        """
        Initialize the CleanContext.  Saves the list of objects for
        which temporary dirty detection must be disabled.
        """

        self.objs = objs
        self.makedirty = {}

    def __enter__(self):
        """
        Enter the context.  Turns off ``makedirty`` for all objects,
        saving the original values for later restoration.
        """

        for obj in self.objs:
            self.makedirty[id(obj)] = ResourceObject.makedirty(obj, False)

        # Return the list of objects
        return obj

    def __exit__(self, exc_type, exc_value, exc_tb):
        """
        Exit the context.  Restores the ``makedirty`` for all objects.
        """

        for obj in self.objs:
            ResourceObject.makedirty(obj, self.makedirty[id(obj)])


def cleanaccess(*objs):
    """
    Allows access to the objects given as arguments without affecting
    their ``dirty`` flags.
    """

    return CleanContext(objs)


def dirty(*objs):
    """
    Marks the objects given as arguments as dirty.
    """

    for obj in objs:
        ResourceObject.dirty(obj, True)


def clean(*objs):
    """
    Marks the objects given as arguments as clean.
    """

    for obj in objs:
        ResourceObject.dirty(obj, False)


def getobject(obj):
    """
    The resources system uses proxy objects, so it can catch actions
    which make a resource "dirty".  Sometimes, this interferes with
    the action of the base resource object, for instance, when the
    base resource object implements one of the special methods.  This
    function provides a means of accessing the base resource object
    directly, given a wrapped resource object ``obj``.
    """

    return ResourceObject.obj(obj)


class Resource(object):
    """
    Resource class, which describes test resources.  To define a
    resource, extend this class and implement the setUp() method and,
    optionally, the tearDown() method.  Subclasses may also specify
    alternate values for the ``oneshot`` and ``dirtymeths`` class
    attributes: if ``oneshot`` is True, every acquired resource will
    only be used once; and ``dirtymeths`` should be a list giving the
    name of methods which, when called, cause the resource object to
    be considered dirty.
    """

    # If set to True, resource will never be used more than once
    oneshot = False
    dirtymeths = []

    def __init__(self, *args, **kwargs):
        """
        Initialize a Resource.  This saves the arguments, which will
        later be passed to setUp().
        """

        # Build a key from the arguments
        key = [self.__class__]
        key += [str(a) for a in args]
        key += ['%s=%s' % (k, kwargs[k]) for k in sorted(kwargs.keys())]
        self.key = tuple(key)

        # Save the arguments
        self.args = args
        self.kwargs = kwargs

    def acquire(self):
        """
        Acquire a resource object.  This calls the setUp() method and
        binds its return result to this resource object using the
        ResourceObject class.
        """

        # Build and return the actual resource object
        obj = self.setUp(*self.args, **self.kwargs)

        # Build our proxy object
        return ResourceObject(self, obj, self.dirtymeths)

    def release(self, obj, msgs, status=None, force=False):
        """
        Release a resource object.  If the object cannot be reused, or
        if ``force`` is True, the tearDown() method will be called.
        Returns True if the object may be reused, otherwise returns
        False.
        """

        # Do we need to release the object?
        if force or self.oneshot or ResourceObject.dirty(obj):
            try:
                self.tearDown(ResourceObject.obj(obj), status)
            except:
                # In the event of an error releasing a resource, save
                # a message documenting the failure
                msgs.append((self, sys.exc_info()))
            return False

        return True

    def setUp(self, *args, **kwargs):
        """
        Sets up and returns the resource.  Must be implemented by all
        subclasses.
        """

        raise NotImplementedError("%s.%s.setUp() is not implemented" %
                                  (self.__class__.__module__,
                                   self.__class__.__name__))

    def tearDown(self, obj, status):
        """
        Tears down a resource allocated by setUp().  This is optional;
        implement it only if you need to release resources, such as
        open files.

        Note that the test's status will be passed as the ``status``
        argument, except in the case of cached resources being cleaned
        up after all tests have finished running.  This status value
        can be used to ensure that debugging resources, such as
        temporary files, are preserved in the event of a failure.
        """

        pass


class ResourceManager(object):
    """
    ResourceManager class, which manages a pool of resources.
    """

    def __init__(self):
        """
        Initializes the resource pool.
        """

        self._pool_lock = semaphore.Semaphore()
        self._pool = {}

        # Need a place to store error messages
        self._messages = []

    def _get_pool(self, res):
        """
        Retrieve the pool corresponding to the resource ``res``.  This
        method must be called with the pool lock held.
        """

        # Make sure we have the pool...
        if res.key not in self._pool:
            self._pool[res.key] = []

        # Return the resource pool
        return self._pool[res.key]

    def acquire(self, res):
        """
        Acquire a resource object corresponding to the resource
        ``res``.
        """

        # Hold the lock while we're accessing the resource pool
        with self._pool_lock:
            pool = self._get_pool(res)

            # Do we have an available resource?
            if len(pool) > 0:
                return pool.pop(0)

        # OK, create a new resource object
        return res.acquire()

    def release(self, obj, status=None):
        """
        Release a resource object ``obj``.  If the object is dirty, it
        will be discarded; otherwise, it will be added to the resource
        pool for later reuse.
        """

        # Get the resource
        res = ResourceObject.resource(obj)

        # Let the resource do any cleaning up it needs to do...
        if not res.release(obj, self._messages, status=status):
            # It was dirty, so we got rid of it
            return

        # OK, we're going to add it back to the resource pool, so grab
        # the lock
        with self._pool_lock:
            # Get the pool
            pool = self._get_pool(res)

            # Append resource object to the end of the pool, so we
            # have FIFO-style reuse
            pool.append(obj)

    def release_all(self):
        """
        Release all resources in the resource pool.
        """

        # Force-release all resource objects
        with self._pool_lock:
            for objlist in self._pool.values():
                for obj in objlist:
                    res = ResourceObject.resource(obj)
                    res.release(obj, self._messages, force=True)

            # Clear the pool
            self._pool = {}

    def collect(self, resources):
        """
        Collects the resources identified by the ``resources``
        dictionary and yields them as a dictionary to the caller.  The
        resources allocated are then released when the generator
        continues.  The generator's send() method should be called
        with the test status, which will then be passed to the
        resource tearDown() methods.
        """

        # Set up the resources we need...
        objects = {}
        for key, res in resources.items():
            objects[key] = self.acquire(res)

        # Yield the resource dictionary and get the test status
        status = yield objects

        # Now, release the resources we used
        for obj in objects.values():
            self.release(obj, status=status)

    @property
    def messages(self):
        """
        Retrieve all error messages accumulated while releasing
        resources.  Clears the message list.
        """

        # Get the current messages
        msgs = self._messages

        # Clear the list
        self._messages = []

        return msgs



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import dtest

# Need t_order
import tests


def setUp():
    # Make sure we're the third thing to have run
    assert len(tests.t_order) == 2, "Ordering error running test suite"
    assert tests.t_order[-1] == 'tests.ordering.setUp', \
        "Incorrect previous step"

    # Keep track of what has run
    tests.t_order.append('tests.ordering.test_order.setUp')


def tearDown():
    # Make sure we're the twelfth thing to have run
    assert len(tests.t_order) == 11, "Ordering error running test suite"
    assert tests.t_order[-1] == ('tests.ordering.test_order.'
                                 'OrderingTestCase.tearDownClass'), \
                                 "Incorrect previous step"

    # Keep track of what has run
    tests.t_order.append('tests.ordering.test_order.tearDown')


class OrderingTestCase(dtest.DTestCase):
    @classmethod
    def setUpClass(cls):
        # Make sure we're the fourth thing to have run
        assert len(tests.t_order) == 3, "Ordering error running test suite"
        assert tests.t_order[-1] == 'tests.ordering.test_order.setUp', \
            "Incorrect previous step"

        # Keep track of what has run
        tests.t_order.append('tests.ordering.test_order.'
                             'OrderingTestCase.setUpClass')

    @classmethod
    def tearDownClass(cls):
        # Make sure we're the eleventh thing to have run
        assert len(tests.t_order) == 10, "Ordering error running test suite"
        assert tests.t_order[-1] == ('tests.ordering.test_order.'
                                     'OrderingTestCase.tearDown'), \
                                     "Incorrect previous step"

        # Keep track of what has run
        tests.t_order.append('tests.ordering.test_order.'
                             'OrderingTestCase.tearDownClass')

    def setUp(self):
        # Make sure we're the fifth or eighth thing to have run
        assert len(tests.t_order) == 4 or len(tests.t_order) == 7, \
            "Ordering error running test suite"
        if len(tests.t_order) == 4:
            assert tests.t_order[-1] == ('tests.ordering.test_order.'
                                         'OrderingTestCase.setUpClass'), \
                                         "Incorrect previous step"
        else:
            assert tests.t_order[-1] == ('tests.ordering.test_order.'
                                         'OrderingTestCase.tearDown'), \
                                         "Incorrect previous step"

        # Keep track of what has run
        tests.t_order.append('tests.ordering.test_order.'
                             'OrderingTestCase.setUp')

    def tearDown(self):
        # Make sure we're the seventh or tenth thing to have run
        assert len(tests.t_order) == 6 or len(tests.t_order) == 9, \
            "Ordering error running test suite"
        if len(tests.t_order) == 6:
            assert tests.t_order[-1] == ('tests.ordering.test_order.'
                                         'OrderingTestCase.test1'), \
                                         "Incorrect previous step"
        else:
            assert tests.t_order[-1] == ('tests.ordering.test_order.'
                                         'OrderingTestCase.test2'), \
                                         "Incorrect previous step"

        # Keep track of what has run
        tests.t_order.append('tests.ordering.test_order.'
                             'OrderingTestCase.tearDown')

    def test1(self):
        # Make sure we're the sixth thing to have run
        assert len(tests.t_order) == 5, "Ordering error running test suite"
        assert tests.t_order[-1] == ('tests.ordering.test_order.'
                                     'OrderingTestCase.setUp'), \
                                     "Incorrect previous step"

        # Keep track of what has run
        tests.t_order.append('tests.ordering.test_order.'
                             'OrderingTestCase.test1')

    @dtest.depends(test1)
    def test2(self):
        # Make sure we're the ninth thing to have run
        assert len(tests.t_order) == 8, "Ordering error running test suite"
        assert tests.t_order[-1] == ('tests.ordering.test_order.'
                                     'OrderingTestCase.setUp'), \
                                     "Incorrect previous step"

        # Keep track of what has run
        tests.t_order.append('tests.ordering.test_order.'
                             'OrderingTestCase.test2')



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
===============================
Dependency-based Test Framework
===============================

The dtest package defines a dependency-based test framework similar to
the standard unittest in the Python standard library.  The primary
advantage of a dependency-based test framework is that it is easy to
run tests in multiple threads, making test runs faster because tests
are performed simultaneously.  It is also possible to ensure that some
tests are skipped if other tests fail, perhaps because the tests to be
skipped are dependent on the very functionality that has been shown to
be improperly implemented.  These dependencies are also used, under
the hood, to safely permit the running of test fixtures at class-,
module-, and package-levels, without worrying about multi-threading
issues.

The dtest framework provides a DTestCase class, similar to
unittest.TestCase.  There are also a number of decorators available,
to do such things as: marking a function or method as being (or not
being) a test (@istest and @nottest); marking a test to be skipped by
default (@skip); marking a test as having an expected failure
(@failing); setting arbitrary attributes on a test (@attr());
indicating that a test is dependent on other tests (@depends());
indicating that a test is expected to raise a given exception or one
of a given set of exceptions (@raises()); marking that a test should
conclude within a given time limit (@timed()); requesting that a test
be executed multiple times (@repeat()); setting an alternate
parallelization strategy (@strategy()); using the multithreaded
parallelization strategies (@parallel()); setting the result policy
(@policy()); and using the threshold result policy (@threshold()).

Tests may be discovered using the explore() function, which returns an
instance of DTestQueue.  (This instance may be passed to other
invocation of explore(), to discover tests in multiple directories.)
Once tests have been discovered, a dependency graph may then be
generated using the DTestQueue.dot() method, or the test suite may be
executed by calling the DTestQueue.run().

It is possible to capture arbitrary forms of output by extending and
instantiating the Capturer class.  Note that standard output and
standard error are captured by default.  Capturing may be disabled for
a test run by passing a True ``debug`` argument to the
DTestQueue.run() method.

Tests themselves may be written using the ``assert`` statement, if
desired, but a number of utilities are available in the dtest.util
package for performing various common tests.  Additionally, a special
output stream, ``dtest.status``, is provided; this stream may be used
to emit status messages to inform the user of the status of a
long-running test.  (Additional properties and methods are available
on ``dtest.status`` for supporting this special stream within
newly-created threads.  The built-in parallelization strategies
already use this support.  For more information, see the documentation
for ``dtest.status.output``, ``dtest.status.test``, and
``dtest.status.setup()``.)

For complex testing behavior, generator test functions are possible.
These test functions should yield either a callable or a tuple.  If a
tuple is yielded, the first or second element must be a callable, and
the elements after the callable identify positional arguments, keyword
arguments, or both (in the order positional arguments as a sequence,
followed by keyword arguments as a dictionary).  If the callable is
the second element of the tuple, the first must be a string giving a
name for the test.  Note that yielded tests cannot have dependencies,
fixtures or any of the other DTest decorators; all such enhancements
must be attached to the generator function; on the other hand, it is
legal for the yielded callable to be a generator itself, which will be
treated identically to the top-level generator function.  Also note
that when the @repeat() decorator is applied to a generator test
function, each yielded function will be called the designated number
of times, but the generator itself will be called only once.

When using the above complex testing behavior, it is also possible to
affect the overall result based on the number of individual successes,
failures, and errors encountered through the use of result policies.
By default, the overall result is only a success if all individual
runs return result, and an error overall result is reported if any
errors were encountered; however, a threshold policy can be selected
by decorating the test with the @threshold() decorator.  In the
threshold policy, any errors result in an overall error result, but
only a given percentage of tests must succeed in order for the overall
result to be a success.  It is also possible to build special-purpose
result policies; they can be attached to a test using the @policy()
decorator.

Note that both dtest and dtest.util are safe for use with "import *".
"""

from dtest.capture import Capturer
from dtest.constants import *
from dtest.exceptions import DTestException
from dtest.core import DTestQueue, DTestOutput, status, explore, main, \
    optparser, opts_to_args
from dtest.resource import cleanaccess, dirty, clean, getobject, \
    Resource
from dtest.test import istest, nottest, isfixture, skip, failing, attr, \
    depends, raises, timed, repeat, strategy, parallel, policy, threshold, \
    require, DTestCase

__all__ = ['Capturer',
           'PRE', 'POST', 'TEST',
           'RUNNING', 'FAIL', 'XFAIL', 'ERROR', 'DEPFAIL', 'OK', 'UOK',
           'SKIPPED',
           'DTestException',
           'DTestQueue', 'DTestOutput', 'status', 'explore', 'main',
           'optparser', 'opts_to_args',
           'cleanaccess', 'dirty', 'clean', 'getobject', 'Resource',
           'istest', 'nottest', 'isfixture', 'skip', 'failing', 'attr',
           'depends', 'raises', 'timed', 'repeat', 'strategy', 'parallel',
           'policy', 'threshold', 'require', 'DTestCase']



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
============
Test Results
============

This module contains the DTestResult and DTestMessage classes, which
are used to represent the results of tests.  Instances of DTestResult
contain the current state of a test, whether the test was successful
or if an error was encountered, and any exceptions and output messages
that were generated while running the test.  The output messages are
contained in an instance of DTestMessage.
"""

from dtest import capture
from dtest.constants import *

from eventlet.timeout import Timeout


class ResultContext(object):
    """
    ResultContext
    =============

    The ResultContext class is a Python context manager used in the
    automatic collection of captured output and exception handling.
    It is instantiated by DTestResult.accumulate()
    """

    def __init__(self, result, ctx, excs):
        """
        Initialize a ResultContext associated with the given
        ``result``.  The context will handle messages for the part of
        the test run given by ``ctx`` (may be PRE, TEST, or POST).
        The exceptions listed in the ``excs`` tuple are expected; if
        no exceptions are expected, pass ``excs`` as None.
        """

        # Save the basic information
        self.result = result
        self.ctx = ctx
        self.excs = excs

        # There's no timeout...
        self.timeout = None

    def __enter__(self):
        """
        Begin the context handling.  Clears out any captured data and
        initializes any timeouts defined for the test.
        """

        # Clear the captured values for this thread
        capture.retrieve()

        # If test should be timed, set up the timeout
        if self.result._test._timeout:
            self.timeout = Timeout(self.result._test._timeout,
                                   AssertionError("Timed out after %s "
                                                  "seconds" %
                                                  self.result._test._timeout))

    def __exit__(self, exc_type, exc_value, tb):
        """
        Ends context handling.  Cancels any pending timeouts,
        retrieves output data and exceptions, and determines the final
        result of the test.  A DTestMessage object is initialized if
        necessary.
        """

        # Cancel the timeout if one is pending
        if self.timeout is not None:
            self.timeout.cancel()
            self.timeout = None

        # Get the output and clean up
        captured = capture.retrieve()

        # If this was the test, determine a result
        if self.ctx in (PRE, TEST):
            self.result._set_result(self, exc_type, exc_value, tb)

        # Generate a message, if necessary
        if captured or exc_type or exc_value or tb:
            self.result._storemsg(self, captured, exc_type, exc_value, tb)

        # We handled the exception
        return True

class DTestResult(object):
    """
    DTestResult
    ===========

    The DTestResult class stores the current state of a test, as well
    as the results and output messages of a test and its immediate
    fixtures.  Various special methods are implemented, allowing the
    result to appear True if the test passed and False if the test did
    not pass, as well as allowing the messages to be accessed easily.
    Three public properties are available: the ``test`` property
    returns the associated test; the ``state`` property returns the
    state of the test, which can also indicate the final result; and
    the ``msgs`` property returns a list of the messages generated
    while executing the test.

    Note that the string representation of a DTestResult object is
    identical to its state.

    Test messages
    -------------

    Messages can be emitted during three separate phases of test
    execution.  The first step of executing a test is to execute the
    setUp() method defined for the class; the second step is executing
    the test itself; and the third step is to execute the tearDown()
    method defined for the class.  (The setUp() and tearDown() methods
    used can be set or overridden using the setUp() and tearDown()
    decorators of DTest.)  Messages produced by each phase are saved,
    and are identified by the constants PRE, TEST, and POST,
    respectively.  This could be used to warn a developer that,
    although a test passed, the following tearDown() function failed
    for some reason.

    The list of test message objects (instances of class DTestMessage)
    can be retrieved, in the order (PRE, TEST, POST), using the
    ``msgs`` property, as indicated above.  Additionally, the presence
    of each type of message can be discerned with the ``in`` operator
    (e.g., ``PRE in result``), and the message itself retrieved using
    array accessor syntax (e.g., ``result[TEST]``).  The total number
    of messages available can be determined using the len() operator.
    """

    def __init__(self, test):
        """
        Initialize a DTestResult object corresponding to the given
        ``test``.
        """

        self._test = test
        self._state = None
        self._result = None
        self._error = False
        self._msgs = {}

    def __nonzero__(self):
        """
        Allows a DTestResult object to be used in a boolean context;
        the object will test as True if the test passed, otherwise it
        will test as False.
        """

        # The boolean value is True for pass, False for fail or not
        # run
        return self._result is True

    def __len__(self):
        """
        Allows the len() built-in to be called on a DTestResult
        object.  Returns the number of messages.
        """

        # Return the number of messages
        return len(self._msgs)

    def __getitem__(self, key):
        """
        Allows a message, as specified by ``key``, to be retrieved
        using array access notation (square brackets, "[" and "]").
        Valid values for ``key`` are the constants PRE, TEST, and
        POST.
        """

        # Return the message for the desired key
        return self._msgs[key]

    def __contains__(self, key):
        """
        Allows the ``in`` operator to be used on a DTestResult object.
        Determines if the message specified by ``key`` is set on this
        result.  Valid values for ``key`` are the constants PRE, TEST,
        and POST.
        """

        # Does the key exist in the list of messages?
        return key in self._msgs

    def __str__(self):
        """
        Allows the str() built-in to be called on a DTestResult
        object.  Returns the string version of the test state.  In the
        event the test has not been run, returns the empty string.
        """

        # Return our state, which is an excellent summary of the
        # result
        return '' if self._state is None else self._state

    def __repr__(self):
        """
        Allows the repr() built-in to be called on a DTestResult
        object.  Augments the default representation to include the
        state and the messages present.
        """

        # Generate a representation of the result
        return ('<%s.%s object at %#x state %s with messages %r>' %
                (self.__class__.__module__, self.__class__.__name__,
                 id(self), self._state, self._msgs.keys()))

    def _transition(self, state=None, output=None):
        """
        Performs a transition to the given ``state``.  If ``state`` is
        None, the state will be determined from the status of the
        ``_result`` and ``_error`` attributes, set by __exit__().
        Note that the test's ``_exp_fail`` attribute is also consulted
        to determine if the result was expected or not.
        """

        # If state is None, determine the state to transition to based
        # on the result
        if state is None:
            if self._result:
                state = UOK if self._test._exp_fail else OK
            elif self._error:
                state = ERROR
            else:
                state = XFAIL if self._test._exp_fail else FAIL

        # Issue an appropriate notification
        if output is not None:
            output.notify(self._test, state)

        # Transition to the new state
        self._state = state

    def _set_result(self, ctx, exc_type, exc_value, tb):
        """
        Determines the result or error status of the test.  Only
        called if the context is PRE or TEST.
        """

        # Are we expecting any exceptions?
        if ctx.excs:
            self._result = exc_type in ctx.excs
            self._error = (exc_type not in ctx.excs and
                           exc_type != AssertionError)
        else:
            # Guess we're not...
            self._result = exc_type is None
            self._error = (exc_type is not None and
                           exc_type != AssertionError)

    def _storemsg(self, ctx, captured, exc_type, exc_value, tb):
        """
        Allocates and stores a DTestMessage instance which brings
        together captured output and exception values.
        """

        self._msgs[ctx.ctx] = DTestMessage(ctx.ctx, captured,
                                           exc_type, exc_value, tb)

    def accumulate(self, nextctx, excs=None, id=None):
        """
        Prepares a context manager for accumulating output for a
        portion of a test.  The ``nextctx`` argument must be one of
        the constants PRE, TEST, or POST, indicating which phase of
        test execution is about to occur.  If ``excs`` is not None, it
        should be a tuple of the exceptions to expect the execution to
        raise; the test passes if one of these exceptions is raised,
        or fails otherwise.
        """

        # Return a context for handling the result
        return ResultContext(self, nextctx, excs)

    @property
    def test(self):
        """
        Retrieve the test associated with this DTestResult object.
        """

        # We want the test to be read-only, but to be accessed like an
        # attribute
        return self._test

    @property
    def state(self):
        """
        Retrieve the current state of this DTestResult object.  If the
        test has not been executed, returns None.
        """

        # We want the state to be read-only, but to be accessed like
        # an attribute
        return self._state

    @property
    def msgs(self):
        """
        Retrieve the list of messages associated with this DTestResult
        object.  The tests will be in the order (PRE, TEST, POST); if
        a given message does not exist, it will be omitted from the
        list.
        """

        # Retrieve the messages in order
        msglist = []
        for mt in (PRE, TEST, POST):
            if mt in self._msgs:
                msglist.append(self._msgs[mt])

        # Return the list of messages
        return msglist

    @property
    def multi(self):
        """
        Returns True only if the result is a multi-result.
        """

        return False


class DTestMessage(object):
    """
    DTestMessage
    ============

    The DTestMessage class is a simple container class for messages
    generated by test execution.  The following attributes are
    defined:

    :ctx:
        The context in which the message was generated.  May be one of
        the constants PRE, TEST, or POST.

    :captured:
        A list of tuples containing captured output.  For each tuple,
        the first element is a short name; the second element is a
        description, suitable for display to the user; and the third
        element is the captured output.  All three elements are simple
        strings.

    :exc_type:
        If an unexpected exception (including AssertionError) is
        thrown while executing the test, this attribute will contain
        the type of the exception.  If no exception is thrown, this
        attribute will be None.

    :exc_value:
        If an unexpected exception (including AssertionError) is
        thrown while executing the test, this attribute will contain
        the actual exception object.  If no exception is thrown, this
        attribute will be None.

    :exc_tb:
        If an unexpected exception (including AssertionError) is
        thrown while executing the test, this attribute will contain
        the traceback object.  If no exception is thrown, this
        attribute will be None.
    """

    def __init__(self, ctx, captured, exc_type, exc_value, exc_tb):
        """
        Initialize a DTestMessage object.  See the class docstring for
        the meanings of the parameters.
        """

        # Save all the message information
        self.ctx = ctx
        self.captured = captured
        self.exc_type = exc_type
        self.exc_value = exc_value
        self.exc_tb = exc_tb


class MultiResultContext(ResultContext):
    """
    MultiResultContext
    ==================

    The MultiResultContext class is an extension of the ResultContext
    context manager that adds the requisite support for message IDs.
    These are needed to differentiate the results of multiple test
    runs with DTestResultMulti.
    """

    def __init__(self, result, ctx, excs, msgid):
        """
        Initialize a MultiResultContext associated with the given
        ``result``.  The additional argument ``msgid`` is an ID to be
        associated with the messages that are generated as a result of
        the run; it may be None only if ``ctx`` is TEST.
        """

        # Initialize the superclass
        super(MultiResultContext, self).__init__(result, ctx, excs)

        # Save the message ID as well
        self.msgid = msgid


class DTestResultMulti(DTestResult):
    """
    DTestResultMulti
    ================

    The DTestResultMulti class is an extension of the DTestResult
    class which additionally provides the ability to store the results
    from multiple tests.  This is used, for example, when a defined
    test is a generator, to store the results from all generated
    functions.
    """

    def __init__(self, test):
        """
        Initialize a DTestResultMulti object corresponding to the
        given ``test``.
        """

        super(DTestResultMulti, self).__init__(test)

        # Pre-allocate the KeyedSequence, so we don't run into race
        # conditions
        self._msgseq = KeyedSequence()

        # Also keep track of IDs we've seen so the generated message
        # IDs are stable
        self._idseen = set()

        # Also need to count successes, failures, and errors
        self._success_cnt = 0
        self._failure_cnt = 0
        self._error_cnt = 0
        self._total_cnt = 0

    def _set_result(self, ctx, exc_type, exc_value, tb):
        """
        Extends the superclass method to support threshold-style final
        result computation.
        """

        # If we're in PRE, defer to the superclass method
        if ctx.ctx == PRE:
            super(DTestResultMulti, self)._set_result(ctx, exc_type,
                                                      exc_value, tb)
            return

        # Figure out if this is a success, failure, or an error
        result = None
        if ctx.excs:
            if exc_type in ctx.excs:
                result = '_success_cnt'
        else:
            if exc_type is None:
                result = '_success_cnt'
        if result is None:
            if exc_type != AssertionError:
                result = '_error_cnt'
            else:
                result = '_failure_cnt'

        # Keep track of the number of successes, failures, and errors
        setattr(self, result, getattr(self, result) + 1)
        self._total_cnt += 1

        # Finally, compute the values of _result and _error based on
        # the threshold strategy of the test
        self._result, self._error = self._test._policy(self._total_cnt,
                                                       self._success_cnt,
                                                       self._failure_cnt,
                                                       self._error_cnt)

    def _storemsg(self, ctx, captured, exc_type, exc_value, tb):
        """
        Allocates and stores a DTestMessageMulti instance which brings
        together captured output and exception values.
        """

        # We only specially TEST-context messages
        if ctx.ctx != TEST:
            super(DTestResultMulti, self)._storemsg(ctx, captured, exc_type,
                                                    exc_value, tb)
            return

        # Make sure the message sequence goes in the collection of
        # messages
        if TEST not in self._msgs:
            self._msgs[TEST] = self._msgseq

        # Store a message
        self._msgseq[ctx.msgid] = DTestMessageMulti(ctx.ctx, ctx.msgid,
                                                    captured, exc_type,
                                                    exc_value, tb)

    def accumulate(self, nextctx, excs=None, id=None):
        """
        Prepares the DTestResultMulti object for use as a context
        manager.  The ``nextctx`` argument must be one of the
        constants PRE, TEST, or POST, indicating which phase of test
        execution is about to occur.  If ``excs`` is not None, it
        should be a tuple of the exceptions to expect the execution to
        raise; the test passes if one of these exceptions is raised,
        or fails otherwise.

        The ``id`` parameter must be specified if ``nextctx`` is TEST;
        it identifies the test being executed.
        """

        # Force id to an empty string, if it's not specified
        if not id:
            id = ''

        # Starting with 'id', compute a message ID
        msgid = id
        i = 0
        while msgid in self._idseen:
            i += 1
            msgid = "%s#%d" % (id, i)

        # Mark this message ID as in use
        self._idseen.add(msgid)

        # Return a context for handling the result
        return MultiResultContext(self, nextctx, excs, msgid)

    @property
    def multi(self):
        """
        Returns True only if the result is a multi-result.
        """

        return True


class DTestMessageMulti(DTestMessage):
    """
    DTestMessageMulti
    =================

    The DTestMessageMulti class is an extension of DTestMessage that
    adds an :id: attribute to identify the origin of the message in a
    DTestResultMulti result.
    """

    def __init__(self, ctx, id, captured, exc_type, exc_value, exc_tb):
        """
        Initialize a DTestMessageMulti object.  See the class
        docstring for this class and its superclass for the meanings
        of the parameters.
        """

        # Call the superclass constructor
        super(DTestMessageMulti, self).__init__(ctx, captured, exc_type,
                                                exc_value, exc_tb)

        # Also save the id
        self.id = id


class KeyedSequence(object):
    """
    KeyedSequence
    =============

    The KeyedSequence class is a helper class for DTestResultMulti.
    Messages (DTestMessageMulti objects) from tests are stored in an
    ordered sequence, but the sequence should also be accessible by a
    test name.  This class implements an abbreviated sequence
    interface that enables the sequence to be indexed by an integer
    index (or array slice), or by a string key.  New items may not be
    added to the sequence using the standard operations such as
    append(); they may only be added to the sequence by using
    assignment by string key.

    This class does support iteration.
    """

    def __init__(self):
        """
        Initialize a KeyedSequence object.
        """

        # Keep an index of keys to list positions
        self._index = {}
        self._values = []

    def __contains__(self, key):
        """
        Determines if ``key`` exists within the sequence.  The ``key``
        may only be a string; there is no support for searching for
        items within the sequence.
        """

        # Check if the key exists
        return key in self._index

    def __getitem__(self, key):
        """
        Retrieve the item or items associated with ``key``.  The
        ``key`` may be an integer or a string.  Array slices are also
        supported.
        """

        # If key is an integer or a slice, use the values list
        if isinstance(key, (int, long, slice)):
            return self._values[key]

        # Get the index from the key and return that
        return self._values[self._index[key]]

    def __setitem__(self, key, value):
        """
        Set the item associated with ``key``.  The ``key`` may be an
        integer or a string.  Replacing multiple elements with an
        array slice is not permitted.
        """

        # If key is a slice, fault
        if isinstance(key, slice):
            raise TypeError("cannot replace slice")

        # If key is an integer, use the values list
        if isinstance(key, (int, long)):
            self._values[key] = value
            return

        # Does the key already exist?
        if key not in self._index:
            # Adding a new value, so let's add the key to the index
            self._index[key] = len(self)
            self._values.append(value)
            return

        # OK, just need to replace existing value
        self._values[self._index[key]] = value

    def __len__(self):
        """
        Return the number of items in the KeyedSequence.
        """

        # Return the values
        return len(self._values)

    def __iter__(self):
        """
        Return an iterator which will iterate over the items in the
        sequence.
        """

        # Iterate over the values
        return iter(self._values)

    def count(self, *args, **kwargs):
        """
        Count the number of instances of a particular item in the
        sequence.
        """

        # Use the values list
        return self._values.count(*args, **kwargs)

    def index(self, *args, **kwargs):
        """
        Determine the index of a particular item in the sequence.
        """

        # Use the values list
        return self._values.index(*args, **kwargs)



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
==========================
Parallelization Strategies
==========================

This module contains all the classes necessary for identifying
parallelization strategies.  A parallelization strategy provides
support for alternate modes of parallelizing multiple-result tests,
i.e., tests on which @repeat() has been used or which are generators
providing lists of other test functions to execute.  This module
contains SerialStrategy, UnlimitedParallelStrategy, and
LimitedParallelStrategy.
"""

import dtest

from eventlet import spawn_n
from eventlet.event import Event
from eventlet.semaphore import Semaphore


class SerialStrategy(object):
    """
    SerialStrategy
    ==============

    The SerialStrategy class is a parallelization strategy that causes
    spawned tests to be executed serially, one after another.
    """

    def prepare(self):
        """
        Prepares the SerialStrategy object to spawn a set of tests.
        Since SerialStrategy "spawns" tests by running them
        synchronously, this function is a no-op.
        """

        pass

    def spawn(self, call, *args, **kwargs):
        """
        Spawn a function.  The callable ``call`` will be executed with
        the provided positional and keyword arguments.  Since
        SerialStrategy "spawns" tests by running them synchronously,
        this function simply calls ``call`` directly.
        """

        call(*args, **kwargs)

    def wait(self):
        """
        Waits for spawned tests to complete.  Since SerialStrategy
        "spawns" tests by running them synchronously, this function is
        a no-op.
        """

        pass


class UnlimitedParallelStrategy(object):
    """
    UnlimitedParallelStrategy
    =========================

    The UnlimitedParallelStrategy class is a parallelization strategy
    that causes spawned tests to be executed in parallel, with no
    limit on the maximum number of tests that can be executing at one
    time.
    """

    def prepare(self):
        """
        Prepares the UnlimitedParallelStrategy object to spawn a set
        of tests.  Simply initializes a counter to zero and sets up an
        event to be signaled when all tests are done.
        """

        # Initialize the counter and the event
        self.count = 0
        self.lock = Semaphore()
        self.event = None

        # Save the output and test for the status stream
        self.output = dtest.status.output
        self.test = dtest.status.test

    def spawn(self, call, *args, **kwargs):
        """
        Spawn a function.  The callable ``call`` will be executed with
        the provided positional and keyword arguments.  The ``call``
        will be executed in a separate thread.
        """

        # Spawn our internal function in a separate thread
        self.count += 1
        spawn_n(self._spawn, call, args, kwargs)

    def _spawn(self, call, args, kwargs):
        """
        Executes ``call`` in a separate thread of control.  This
        helper method maintains the count and arranges for the event
        to be signaled when appropriate.
        """

        # Initialize the status stream
        dtest.status.setup(self.output, self.test)

        # Call the call
        call(*args, **kwargs)

        # Decrement the count
        self.count -= 1

        # Signal the event, if necessary
        with self.lock:
            if self.count == 0 and self.event is not None:
                self.event.send()

    def wait(self):
        """
        Waits for spawned tests to complete.
        """

        # Check for completion...
        with self.lock:
            if self.count == 0:
                # No tests still going, so just return
                return

            # OK, let's initialize the event...
            self.event = Event()

        # Now we wait on the event
        self.event.wait()

        # End by clearing the event
        self.event = None


class LimitedParallelStrategy(UnlimitedParallelStrategy):
    """
    LimitedParallelStrategy
    =======================

    The LimitedParallelStrategy class is an extension of the
    UnlimitedParallelStrategy that additionally limits the maximum
    number of threads that may be executing at any given time.
    """

    def __init__(self, limit):
        """
        Initializes a LimitedParallelStrategy object.  The ``limit``
        parameter specifies the maximum number of threads that may
        execute at any given time.
        """

        # Save the limit
        self.limit = limit

    def prepare(self):
        """
        Prepares the LimitedParallelStrategy to spawn a set of tests.
        In addition to the tasks performed by
        UnlimitedParallelStrategy.prepare(), sets up a semaphore to
        limit the maximum number of threads that may execute at once.
        """

        # Call our superclass prepare method
        super(LimitedParallelStrategy, self).prepare()

        # Also initialize a limiting semaphore
        self.limit_sem = Semaphore(self.limit)

    def _spawn(self, call, args, kwargs):
        """
        Executes ``call`` in a separate thread of control.  This
        helper method extends UnlimitedParallelStrategy._spawn() to
        acquire the limiting semaphore prior to executing the call.
        """

        # Call our superclass _spawn method with the limit semaphore
        with self.limit_sem:
            super(LimitedParallelStrategy, self)._spawn(call, args, kwargs)



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from dtest import *
from dtest.util import *


@repeat(2)
def test_multi():
    # Set up a list to record executions
    recorded = []

    # Now, define an inner function
    def inner(*args, **kwargs):
        # Place the arguments into the recorded list
        recorded.append((args, kwargs))

    # Now, yield the inner function once...
    yield ('inner1', inner, (1,), dict(kw=1))

    # Yield it again
    yield ('inner2', inner, (2,), dict(kw=2))

    # Now, check if recorded has what we expect
    assert_equal(len(recorded), 4)
    assert_tuple_equal(recorded[0][0], (1,))
    assert_dict_equal(recorded[0][1], dict(kw=1))
    assert_tuple_equal(recorded[1][0], (1,))
    assert_dict_equal(recorded[1][1], dict(kw=1))
    assert_tuple_equal(recorded[2][0], (2,))
    assert_dict_equal(recorded[2][1], dict(kw=2))
    assert_tuple_equal(recorded[3][0], (2,))
    assert_dict_equal(recorded[3][1], dict(kw=2))



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from dtest import *
from dtest.resource import ResourceObject
from dtest.util import *


class ObjTest(object):
    pass


class ResourceTest(Resource):
    oneshot = True
    def setUp(self, name, value):
        t = ObjTest()
        setattr(t, name, value)
        return t


@require(test1=ResourceTest('foo', 'bar'),
         test2=ResourceTest('bar', 'foo'))
def test_basic(test1, test2):
    assert_is_instance(test1, ResourceObject)
    assert_is_instance(test2, ResourceObject)
    assert_is_instance(getobject(test1), ObjTest)
    assert_is_instance(getobject(test2), ObjTest)
    assert_not_equal(id(test1), id(test2))
    assert_not_equal(id(getobject(test1)), id(getobject(test2)))
    assert_equal(test1.foo, 'bar')
    assert_equal(test2.bar, 'foo')


@require(test=ResourceTest('foo', True))
def test_dirty_direct(test):
    assert_equal(ResourceObject.dirty(test), False)
    dirty(test)
    assert_equal(ResourceObject.dirty(test), True)
    clean(test)
    assert_equal(ResourceObject.dirty(test), False)


@depends(test_dirty_direct)
@require(test=ResourceTest('foo', True))
def test_dirty_attr(test):
    assert_equal(ResourceObject.dirty(test), False)
    test.foo = False
    assert_equal(ResourceObject.dirty(test), True)
    clean(test)
    assert_equal(ResourceObject.dirty(test), False)
    del test.foo
    assert_equal(ResourceObject.dirty(test), True)


@depends(test_dirty_direct)
@require(test=ResourceTest('foo', True))
def test_dirty_access(test):
    assert_equal(ResourceObject.dirty(test), False)
    with cleanaccess(test):
        test.foo = True
    assert_equal(ResourceObject.dirty(test), False)
    with cleanaccess(test):
        del test.foo
    assert_equal(ResourceObject.dirty(test), False)


class ObjTestMeth(object):
    def foo(self):
        self.test = False
    def bar(self):
        self.test = True


class ResourceTestMeth(Resource):
    oneshot = True
    dirtymeths = ['foo']
    def setUp(self):
        t = ObjTestMeth()
        t.test = True
        return t


@depends(test_dirty_direct)
@require(test=ResourceTestMeth())
def test_dirty_meth(test):
    assert_equal(ResourceObject.dirty(test), False)
    assert_equal(test.test, True)
    test.foo()
    assert_equal(ResourceObject.dirty(test), True)
    assert_equal(test.test, False)
    clean(test)
    assert_equal(ResourceObject.dirty(test), False)
    test.bar()
    assert_equal(ResourceObject.dirty(test), False)
    assert_equal(test.test, True)
    with cleanaccess(test):
        test.foo()
    assert_equal(ResourceObject.dirty(test), False)
    assert_equal(test.test, False)


reuse_cache = None


class ReuseObj(object):
    pass


class ResourceTestReuse(Resource):
    def setUp(self, name, value):
        t = ReuseObj()
        setattr(t, name, value)
        return t


@require(test=ResourceTestReuse('foo', True))
def test_reuse1(test):
    global reuse_cache
    reuse_cache = test


@depends(test_reuse1)
@require(test=ResourceTestReuse('foo', True))
def test_reuse2(test):
    assert_equal(reuse_cache, test)


@depends(test_reuse2)
@require(test=ResourceTestReuse('foo', False))
def test_reuse3(test):
    assert_not_equal(reuse_cache, test)


@depends(test_reuse3)
@require(test=ResourceTestReuse('foo', True))
def test_reuse4(test):
    assert_equal(reuse_cache, test)
    dirty(test)


@depends(test_reuse4)
@require(test=ResourceTestReuse('foo', True))
def test_reuse5(test):
    assert_not_equal(reuse_cache, test)


teardown_oneshot_cache = None


class ObjTestTearDown(object):
    pass


class ResourceTestTearDownOneShot(Resource):
    oneshot = True
    def setUp(self, name, value):
        t = ObjTestTearDown()
        setattr(t, name, value)
        return t

    def tearDown(self, obj, result):
        obj.torndown = result


@require(test=ResourceTestTearDownOneShot('foo', True))
def test_teardown1_oneshot(test):
    global teardown_oneshot_cache
    teardown_oneshot_cache = test
    assert_equal(getattr(test, 'torndown', None), None)


@depends(test_teardown1_oneshot)
def test_teardown2_oneshot():
    assert_equal(getattr(teardown_oneshot_cache, 'torndown', None), 'OK')


teardown_dirty_cache = None


class ResourceTestTearDownDirty(Resource):
    def setUp(self, name, value):
        t = ObjTestTearDown()
        setattr(t, name, value)
        return t

    def tearDown(self, obj, result):
        obj.torndown = result


@require(test=ResourceTestTearDownDirty('foo', True))
def test_teardown1_dirty(test):
    global teardown_dirty_cache
    teardown_dirty_cache = test
    dirty(test)
    assert_equal(getattr(test, 'torndown', None), None)


@depends(test_teardown1_dirty)
def test_teardown2_dirty():
    assert_equal(getattr(teardown_dirty_cache, 'torndown', None), 'OK')



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from dtest import *
from dtest.util import *


@threshold(50.0)
def test_threshold():
    # Test function to return
    def tfcn(i):
        # Succeed if i is even
        assert_equal(i % 2, 0)

    # Yield several iterations of tfcn
    for i in range(100):
        yield (tfcn, (i,))



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from dtest import *
from dtest.util import *


# Define setUpClass/tearDownClass/setUp/tearDown for inheritance
class TestInheritanceBase(DTestCase):
    class_setup = None
    instance_setup = None

    @classmethod
    def setUpClass(cls):
        assert_is_none(cls.class_setup)
        cls.class_setup = True

    @classmethod
    def tearDownClass(cls):
        assert_false(cls.class_setup)

    def setUp(self):
        assert_is_none(self.instance_setup)
        self.instance_setup = True

    def tearDown(self):
        assert_false(self.instance_setup)


# See if we inherited them
class TestInheritance(TestInheritanceBase):
    @attr(must_skip=True)
    def test_inheritance(self):
        assert_true(self.class_setup)
        assert_true(self.instance_setup)

        TestInheritanceBase.class_setup = False
        self.instance_setup = False


# Let's really stress things out, here...
class TestInheritanceTwo(TestInheritance):
    def test_inheritance(self):
        # Make sure we can call our superclass method
        super(TestInheritanceTwo, self).test_inheritance()



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
===============
Result Policies
===============

This module contains all the functions and classes necessary for
identifying result policies.  A result policy comes into play only for
tests that result in multiple test calls.  Result policies are simply
callables (functions or objects with __call__() methods) that receive
four counts--the total number of results accumulated so far, the total
number that succeeded, the total number that failed, and the total
number of errors encountered.  They must return a tuple containing two
boolean values--one indicating whether the test is an overall success
(True) or not, and the second indicating whether the test is an error.
(The second may only be True if the first is False.)
"""


def basicPolicy(tot, suc, fail, err):
    """
    Implements the basic policy--all tests must be a success for the
    overall result to be a success, and if there are any errors, the
    overall result is an error.
    """

    return (fail == 0 and err == 0), (err > 0)


class ThresholdPolicy(object):
    """
    ThresholdPolicy
    ===============

    Implements the threshold policy--there must be no errors, and the
    number of successes must exceed a given threshold (expressed as a
    percentage) for the overall result to be a success.
    """

    def __init__(self, threshold):
        """
        Initialize the ThresholdPolicy object.  The ``threshold`` must
        be expressed as a percentage (0 to 100); float values are
        legal here.
        """

        # Save the threshold as a float
        self.threshold = float(threshold)

    def __call__(self, tot, suc, fail, err):
        """
        Implements the threshold policy.  If ``err`` is greater than
        zero, the overall result is an error; otherwise, if ``suc``
        represents more than the configured threshold percentage of
        ``tot``, the overall result is a success.  Note that if
        ``fail`` is zero, the threshold logic is skipped.
        """

        # If there are any errors, we have an error result
        if err > 0:
            return False, True

        # If fail is 0, no point going on
        if fail == 0:
            return True, False

        # Compute the percentage of successes...
        percent = (suc * 100.0) / tot

        # We're successful only if percent is greater than threshold
        return (percent >= self.threshold), False



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Need t_order
import tests


def setUp():
    # Make sure we're the second thing to have run
    assert len(tests.t_order) == 1, "Ordering error running test suite"
    assert tests.t_order[-1] == 'tests.setUp', "Incorrect previous step"

    # Keep track of what has run
    tests.t_order.append('tests.ordering.setUp')


def tearDown():
    # Make sure we're the thirteenth thing to have run
    assert len(tests.t_order) == 12, "Ordering error running test suite"
    assert tests.t_order[-1] == 'tests.ordering.test_order.tearDown', \
        "Incorrect previous step"

    # Keep track of what has run
    tests.t_order.append('tests.ordering.tearDown')



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
=========================
Test Framework Exceptions
=========================

This module contains the DTestException class, which is an extension
raised by the framework when an error is encountered while executing
functions or methods of the framework itself.
"""


class DTestException(Exception):
    """
    DTestException
    ==============

    The DTestException is an exception class used for all exceptions
    generated by the test framework.
    """

    pass



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
=======================
Generic Output Capturer
=======================

This module contains the Capturer class, which can be used to capture
arbitrary output from tests in a thread-safe fashion.  A new Capturer
is defined by subclassing the Capturer class and providing
implementations for the init(), retrieve(), install(), and uninstall()
methods; then, the subclass is instantiated.  Once instantiated,
capturing by the test framework is automatic--registration of the
instance is done by the Capturer constructor.  The module also defines
StdStreamCapturer and produces two instantiations of it, for capturing
stdout and stderr.

Usage
-----

Each Capturer instance must have a unique name, and additionally a
description, which will be output in the test report.  The init()
method of a Capturer instance is called to create a new object to
intercept and store the output; for stream-like Capturer instances,
this could simply return, say, an instantiation of StringIO.  The
retrieve() method will be passed this object and must return a string
consisting of all the output.  The install() and uninstall() methods
cooperate to install a special CaptureProxy object.  For an example of
a full Capturer subclass, check out StdStreamCapturer, contained
within this module::

    class StdStreamCapturer(Capturer):
        def init(self):
            # Create a new StringIO stream
            return StringIO()

        def retrieve(self, st):
            # Retrieve the value of the StringIO stream
            return st.getvalue()

        def install(self, new):
            # Retrieve and return the old stream and install the new one
            old = getattr(sys, self.name)
            setattr(sys, self.name, new)
            return old

        def uninstall(self, old):
            # Re-install the old stream
            setattr(sys, self.name, old)

    # Add capturers for stdout and stderr
    StdStreamCapturer('stdout', 'Standard Output')
    StdStreamCapturer('stderr', 'Standard Error')

Implementation Details
----------------------

The eventlet.corolocal.local class is used to maintain a set of
capturing objects (as initialized by the Capturer.init() method) for
each thread.  The Capturer.install() method is used by the framework
to install a special CaptureProxy object, which uses this thead-local
data to proxy all attribute accesses to the correct capturing object.
Once a test is complete, the captured data is retrieved by calling the
Capturer.retrieve() method, and once all tests have finished, the
Capturer.uninstall() method is used to restore the original values
that the Capturer.install() method discovered when it installed the
CaptureProxy object.

The capture module exports three functions used only by the framework;
these probably should not be called directly by a test author.  The
retrieve() function retrieves the captured data by calling the
Capturer.retrieve() methods in turn; the data is returned in the same
order in which it was registered.

The next two internal functions are install() and uninstall(), which
simply call the Capturer.install() and Capturer.uninstall() methods in
turn.  Note that these calls are not made in any defined order, so
test authors should not rely on any given ordering.

The capture module also pre-defines two Capturer instances, one for
capturing output to sys.stdout, and the other for capturing output to
sys.stderr; the code for this is included in the example above, and
can be referred to while building your own Capturer subclasses.
"""

from StringIO import StringIO
import sys

from eventlet.corolocal import local

from dtest.exceptions import DTestException


# Globals needed for capturing
_installed = False
_saves = {}


class Capturer(object):
    """
    Capturer
    ========

    The Capturer class is an abstract class which keeps track of all
    instances.  It is used to set up new output capturers, in a
    thread-safe way.  Subclasses must implement the init(),
    retrieve(), install(), and uninstall() methods.  The Capturer
    class should not be instantiated directly, as these necessary
    methods are unimplemented.

    Two class variables are defined; the _capturers dictionary stores
    a mapping from a Capturer ``name`` to a Capturer instance, while
    the _caporder list contains a list of Capturer ``name``s in the
    order in which they were instantiated.
    """

    _capturers = {}
    _caporder = []

    def __new__(cls, name, desc):
        """
        Allocate and initialize a Capturer.  Each instance of Capturer
        must have a unique ``name``; this name permits distinct
        Capturer instances to be looked up by CaptureProxy instances.
        The ``desc`` argument should contain a short description which
        will be used to indicate the source of the capture in the test
        output.

        If an attempt to reuse a ``name`` is made, the previous
        instance of that name will be returned, rather than a new one
        being instantiated.
        """

        # First, make sure name isn't already in use
        if name in cls._capturers:
            return cls._capturers[name]

        # Don't allow new capturer registrations after we're installed
        if _installed:
            raise DTestException("Capturers have already been installed")

        # OK, construct a new one
        cap = super(Capturer, cls).__new__(cls)
        cap.name = name
        cap.desc = desc

        # Save it in the cache
        cls._capturers[name] = cap
        cls._caporder.append(name)

        # And return it
        return cap

    def init(self):
        """
        Initialize a Capturer object.  Should return an object which
        exports the appropriate interface for the output being
        intercepted.  All subclasses must implement this method.
        """

        # Initialize a capturer; returns an object that looks like
        # whatever's being captured, but from which a value can later
        # be retrieved.
        raise DTestException("%s.%s.init() unimplemented" %
                             (self.__class__.__module__,
                              self.__class__.__name__))

    def retrieve(self, captured):
        """
        Retrieve data from a Capturer object.  The ``captured``
        argument will be an object returned by the init() method.
        Should return a string consisting of the output data.  All
        subclasses must implement this method.
        """

        # Retrieve the value of a capturer; takes the object returned
        # by init() and returns its string value.
        raise DTestException("%s.%s.retrieve() unimplemented" %
                             (self.__class__.__module__,
                              self.__class__.__name__))

    def install(self, new):
        """
        Install a CaptureProxy object.  The ``new`` argument will be a
        CaptureProxy object, which will delegate all accesses to an
        appropriate object returned by the init() method.  Should
        return the old value of whatever interface is being captured.
        All subclasses must implement this method.
        """

        # Install the capture proxy specified by new; should place
        # that object into the appropriate place so that it can
        # capture output.  Should return the old value, which will
        # later be passed to uninstall.
        raise DTestException("%s.%s.install() unimplemented" %
                             (self.__class__.__module__,
                              self.__class__.__name__))

    def uninstall(self, old):
        """
        Uninstall a CaptureProxy object.  The ``old`` argument will be
        a value returned by the install() method.  The CaptureProxy
        object installed by install() should be uninstalled and
        replaced by the original object specified by ``old``.  All
        subclasses must implement this method.
        """

        # Uninstall the capture proxy by replacing it with the old
        # value specified.  The old value will be the value returned
        # by install().
        raise DTestException("%s.%s.uninstall() unimplemented" %
                             (self.__class__.__module__,
                              self.__class__.__name__))


class _CaptureLocal(local):
    """
    _CaptureLocal
    =============

    The _CaptureLocal class extends eventlet.corolocal.local to
    provide thread-local data.  Its attributes map to objects returned
    by the init() methods of the corresponding Capturer instances, and
    are unique to each thread.
    """

    def __init__(self):
        """
        Initialize a _CaptureLocal object in each thread.  For each
        defined Capturer instance, calls the init() method of that
        object and stores it in an attribute with the same name as the
        Capturer.  This is the magic that allows CaptureProxy to send
        output to the correct place.
        """

        # Walk through all the capturers and initialize them
        for cap in Capturer._capturers.values():
            setattr(self, cap.name, cap.init())


_caplocal = _CaptureLocal()


def retrieve():
    """
    Retrieve captured output for the current thread.  Returns a list
    of tuples, in the same order in which the corresponding Capturer
    instances were allocated.  Each tuple contains the Capturer name,
    its description, and the captured output.  The capture objects are
    reinitialized by this function.
    """

    # Walk through all the capturers and retrieve their description
    # and value
    vals = []
    for name in Capturer._caporder:
        # Get the capturer
        cap = Capturer._capturers[name]

        # Get the current value of the capturer and re-initialize it
        val = cap.retrieve(getattr(_caplocal, name))
        setattr(_caplocal, name, cap.init())

        # Push down the value and other important data
        if val:
            vals.append((cap.name, cap.desc, val))

    # Return the captured values
    return vals


class CaptureProxy(object):
    """
    CaptureProxy
    ============

    The CaptureProxy class delegates all attribute accesses to a
    thread-specific object initialized by the Capturer.init() method.
    The only local attribute of a CaptureProxy object is the _capname
    attribute, which stores the name of the Capturer the CaptureProxy
    is acting on behalf of.  CaptureProxy objects are to be installed
    by the Capturer.install() method and uninstalled by the
    Capturer.uninstall() method.
    """

    def __init__(self, capname):
        """
        Initialize a CaptureProxy by storing the Capturer name.
        """

        # Save the capturer name of interest
        super(CaptureProxy, self).__setattr__('_capname', capname)

    def __getattr__(self, attr):
        """
        Delegate attribute accesses to the proxied, thread-specific
        capturing object.
        """

        # Proxy out to the appropriate object
        return getattr(getattr(_caplocal, self._capname), attr)

    def __setattr__(self, attr, value):
        """
        Delegate attribute updates to the proxied, thread-specific
        capturing object.
        """

        # Proxy out to the appropriate object
        return setattr(getattr(_caplocal, self._capname), attr, value)

    def __delattr__(self, attr):
        """
        Delegate attribute deletions to the proxied, thread-specific
        capturing object.
        """

        # Proxy out to the appropriate stream
        return delattr(getattr(_caplocal, self._capname), attr)


def install():
    """
    Install CaptureProxy objects for all defined Capturer instances.
    For each Capturer instance, the install() method will be called.
    """

    global _installed

    # Do nothing if we're already installed
    if _installed:
        return

    # Remember that we've been installed
    _installed = True

    # Perform the install
    for cap in Capturer._capturers.values():
        _saves[cap.name] = cap.install(CaptureProxy(cap.name))


def uninstall():
    """
    Uninstall CaptureProxy objects for all defined Capturer instances.
    For each Capturer instance, the uninstall() method will be called.
    """

    global _installed
    global _saves

    # Do nothing if we haven't been installed
    if not _installed:
        return

    # Restore our saved objects
    for cap in Capturer._capturers.values():
        cap.uninstall(_saves[cap.name])

    # Reset our state
    _saves = {}
    _installed = False


class StdStreamCapturer(Capturer):
    """
    StdStreamCapturer
    =================

    The StdStreamCapturer is a subclass of Capturer defined to capture
    the standard output and error streams, sys.stdout and sys.stderr,
    respectively.  Output is captured using a StringIO().
    """

    def init(self):
        """
        Initialize a Capturer object.  Returns an instance of
        StringIO.
        """

        # Create a new StringIO stream
        return StringIO()

    def retrieve(self, st):
        """
        Retrieve data from a Capturer object.  The ``st`` argument is
        a StringIO object allocated by the init() method.  Returns a
        string with the contents of the StringIO object.
        """

        # Retrieve the value of the StringIO stream
        return st.getvalue()

    def install(self, new):
        """
        Install a CaptureProxy object.  The ``new`` argument is a
        CaptureProxy object, which is installed in place of sys.stdout
        or sys.stderr (depending on the name used to instantiate the
        Capturer instance).  Returns the original value of the stream
        being replaced.
        """

        # Retrieve and return the old stream and install the new one
        old = getattr(sys, self.name)
        setattr(sys, self.name, new)
        return old

    def uninstall(self, old):
        """
        Uninstall a CaptureProxy object.  The ``old`` argument is the
        original value of the stream, as returned by the install()
        method.  Re-installs that in place of the CaptureProxy object
        installed by install().
        """

        # Re-install the old stream
        setattr(sys, self.name, old)


# Add capturers for stdout and stderr
StdStreamCapturer('stdout', 'Standard Output')
StdStreamCapturer('stderr', 'Standard Error')



#!/usr/bin/python
#
# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from distutils.core import setup

setup(
    name='DTest',
    version='0.5.0',
    description="Dependency-based Threaded Test Framework",
    author="Kevin L. Mitchell",
    author_email="kevin.mitchell@rackspace.com",
    url="http://github.com/klmitch/dtest",
    scripts=['bin/run-dtests'],
    packages=['dtest'],
    license="LICENSE.txt",
    long_description=open('README.rst').read(),
    requires=['eventlet'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Testing',
        ],
    )



#!/usr/bin/python
#
# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
============
Test Running
============

This module contains the run_test() function and the associated Queue
class, which together provide the functionality for executing tests in
a threaded manner while properly handling ordering implied by
dependencies.  Output is specified by passing an instance of
DTestOutput to run().

If this file is executed directly, the main() function--which first
calls explore(), then returns the result of run()--is called, and its
return value will be passed to sys.exit().  Command line arguments are
also available, and the module can be executed by passing "-m
dtest.core" to the Python interpreter.
"""

import imp
from optparse import OptionParser
import os
import os.path
import sys
import traceback

from eventlet import spawn_n, monkey_patch
from eventlet.corolocal import local
from eventlet.event import Event
from eventlet.semaphore import Semaphore

from dtest import capture
from dtest.constants import *
from dtest.exceptions import DTestException
from dtest import resource
from dtest import test


# Default line width
DEF_LINEWIDTH = 78


# Current output for issuing status messages
_output = local()


class _DTestStatus(object):
    """
    _DTestStatus
    ============

    The _DTestStatus class is a stream look-alike class, an instance
    of which implements the special ``dtest.status`` stream.  Data
    written to the stream will be passed to the status() method of the
    current DTestOutput object.  Thread-local data is used to store
    the current DTestOutput object, so multiple output objects may be
    safely used simultaneously.
    """

    def write(self, msg):
        """
        Emits ``msg`` as a status message to the current DTestOutput
        object.  This can be used to notify the user of the status of
        a test which takes a long time to complete.
        """

        # Write to the registered output
        _output.out.status(_output.test, msg)

    def flush(self):
        """
        Provided for compatibility with normal output streams.  Does
        nothing; the DTestOutput object's status() method is assumed
        to perform a flush after every call.
        """

        pass

    @property
    def output(self):
        """
        Retrieve the current DTestOutput object, which is stored in a
        per-thread manner.  This property is provided to allow the
        status stream to be set up in threads started within the
        individual tests.  See the setup() method.
        """

        # This is simple...
        return _output.out

    @property
    def test(self):
        """
        Retrieve the current test object, which is stored in a
        per-thread manner.  This property is provided to allow the
        status stream to be set up in threads started within the
        individual tests.  See the setup() method.
        """

        # Also simple...
        return _output.test

    def setup(self, output, test):
        """
        Initializes the status stream within a new thread of control.
        This routine should be called as the first action of a new
        thread.
        """

        # Set up thread-local data
        _output.out = output
        _output.test = test


# A stream for export
status = _DTestStatus()


class DTestOutput(object):
    """
    DTestOutput
    ===========

    The DTestOutput class is a utility class for grouping together all
    output generation for the test framework.  The ``output``
    attribute contains a stream-like object to which output may be
    sent, and defaults to sys.__stdout__ (note that sys.stdout may be
    captured as the output of a test).  The notify() method is called
    whenever a test or test fixture transitions to an alternate state;
    the result() method is called to output the results of a test; and
    the summary() method is called to output a summary of the results
    of a test.  The default implementations of these methods send
    their output to the stream in the ``output`` attribute, but each
    may be overridden to perform alternate output.  This could, for
    instance, be used to display test framework output in a GUI or to
    generate a web page.
    """

    def __init__(self, output=sys.__stdout__, linewidth=DEF_LINEWIDTH):
        """
        Initialize a DTestOutput object with the given ``output``
        stream (defaults to sys.__stdout__) and linewidth.
        """

        # Save the output and linewidth
        self.output = output
        self.linewidth = linewidth

    def notify(self, test, state):
        """
        Called when a test or test fixture, identified by ``test``,
        transitions to ``state``.  The default implementation ignores
        state transitions by test fixtures or transitions to the
        RUNNING state.
        """

        # Are we interested in this test?
        if not test.istest() or state == RUNNING:
            return

        # Determine the name of the test
        name = str(test)

        # Determine the width of the test name field
        width = self.linewidth - len(state) - 1

        # Truncate the name, if necessary
        if len(name) > width:
            name = name[:width - 3] + '...'

        # Emit the status message
        print >>self.output, "%-*s %s" % (width, name, state)

        # Flush the output
        self.output.flush()

    def result(self, result, debug=False):
        """
        Called at the end of a test run to emit ``result`` information
        for a given test.  Called once for each result.  Should emit
        all exception and captured output information, if any.  Will
        also be called for results from test fixtures, in order to
        emit errors encountered while executing them.  The default
        implementation ignores results containing no messages, and
        only emits results from successful tests if debug is True.
        """

        # Helper for reporting output
        def out_msg(msg, hdr=None):
            # Output header information
            if hdr:
                print >>self.output, (hdr.center(self.linewidth) + "\n" +
                                      ('-' * self.linewidth))

            # Output the test ID
            if hasattr(msg, 'id'):
                id_hdr = " (%s) " % msg.id
                print >>self.output, id_hdr.center(self.linewidth, ':')

            # Output exception information
            if msg.exc_type is not None:
                exc_hdr = ' Exception %s ' % msg.exc_type.__name__
                tb = ''.join(traceback.format_exception(msg.exc_type,
                                                        msg.exc_value,
                                                        msg.exc_tb))
                print >>self.output, exc_hdr.center(self.linewidth, '-')
                print >>self.output, tb.rstrip()

            # Format output data
            for name, desc, value in msg.captured:
                print >>self.output, (' %s ' % desc).center(self.linewidth,
                                                            '-')
                print >>self.output, value.rstrip()

            # Emit a closing line
            print >>self.output, '-' * self.linewidth

        # Skip results with no messages
        if len(result) == 0:
            return

        # If it's successful or an expected failure, only emit
        # messages if debug is True
        if not debug and (result.state == OK or result.state == XFAIL):
            return

        # Emit a banner for the result
        print >>self.output, ("\n" + ("=" * self.linewidth) + "\n" +
                              str(result.test).center(self.linewidth) + "\n" +
                              ("=" * self.linewidth))

        # Emit the data for each step
        if PRE in result:
            out_msg(result[PRE], 'Pre-test Fixture')
        if TEST in result:
            if result.multi:
                for m in result[TEST]:
                    out_msg(m)
            else:
                out_msg(result[TEST])
        if POST in result:
            out_msg(result[POST], 'Post-test Fixture')

        # Flush the output
        self.output.flush()

    def summary(self, counts):
        """
        Called at the end of a test run to emit summary information
        about the run.  The ``counts`` argument is a dictionary
        containing the following keys:

        OK
            The number of tests which passed.  This includes the count of
            unexpected passes (tests marked with the @failing decorator
            which passed).

        UOK
            The number of tests which unexpectedly passed.

        SKIPPED
            The number of tests which were skipped in this test run.

        FAIL
            The number of tests which failed.  This includes the count of
            expected failures (tests marked with the @failing decorator
            which failed).

        XFAIL
            The number of tests which failed, where failure was expected.

        ERROR
            The number of tests which experienced an error--an unexpected
            exception thrown while executing the test.

        DEPFAIL
            The number of tests which could not be executed because tests
            they were dependent on failed.

        'total'
            The total number of tests considered for execution.

        'threads'
            The maximum number of simultaneously executing threads
            which were utilized while running tests.

        Note that test fixtures are not included in these counts.  If a
        test fixture fails (raises an AssertionError) or raises any other
        exception, all tests dependent on that test fixture will fail due
        to dependencies.
        """

        # Emit summary data
        print >>self.output, ("%d tests run in %d max simultaneous threads" %
                              (counts['total'], counts['threads']))
        if counts[OK] > 0:
            unexp = ''
            if counts[UOK] > 0:
                unexp = ' (%d unexpected)' % counts[UOK]
            print >>self.output, ("  %d tests successful%s" %
                                  (counts[OK], unexp))
        if counts[SKIPPED] > 0:
            print >>self.output, "  %d tests skipped" % counts[SKIPPED]
        if counts[FAIL] + counts[ERROR] + counts[DEPFAIL] > 0:
            # Set up the breakdown
            bd = []
            total = 0
            if counts[FAIL] > 0:
                exp = ''
                if counts[XFAIL] > 0:
                    exp = ' [%d expected]' % counts[XFAIL]
                bd.append('%d failed%s' % (counts[FAIL], exp))
                total += counts[FAIL]
            if counts[ERROR] > 0:
                bd.append('%d errors' % counts[ERROR])
                total += counts[ERROR]
            if counts[DEPFAIL] > 0:
                bd.append('%d failed due to dependencies' % counts[DEPFAIL])
                total += counts[DEPFAIL]

            print >>self.output, ("  %d tests failed (%s)" %
                                  (total, ', '.join(bd)))

        # Flush the output
        self.output.flush()

    def caught(self, exc_list):
        """
        Called after emitting summary data to report any exceptions
        encountered within the dtest framework itself while running
        the test.  The ``exc_list`` argument is a list of
        three-element tuples.  For each tuple, the first element is an
        exception type; the second element is the exception value; and
        the third element is a traceback object.  Under most
        circumstances, this function will not be called; if it is, the
        exception data reported should be sent back to the dtest
        framework developers.
        """

        # Emit exception data
        print >>self.output, "\nThe following exceptions were encountered:"
        for exc_type, exc_value, tb in exc_list:
            exc_hdr = ' Exception %s ' % exc_type.__name__
            tb = ''.join(traceback.format_exception(exc_type, exc_value, tb))
            print >>self.output, exc_hdr.center(self.linewidth, '-')
            print >>self.output, tb.rstrip()
        print >>self.output, '-' * self.linewidth
        print >>self.output, ("Please report the above errors to the "
                              "developers of the dtest framework.")

        # Flush the output
        self.output.flush()

    def imports(self, exc_list):
        """
        Called by main() if import errors were encountered while
        discovering tests.  The ``exc_list`` argument is a list of
        tuples containing three elements: the first element is the
        full path to the file for which import was attempted; the
        second element is the module path for which import was
        attempted; and the third is a three-element tuple returned by
        sys.exc_info().
        """

        # Emit import error data
        print >>self.output, "The following import errors were encountered:"
        for path, pkgname, (exc_type, exc_value, tb) in exc_list:
            exc_hdr = ' %s (%s) ' % (os.path.relpath(path), pkgname)
            tb = ''.join(traceback.format_exception(exc_type, exc_value, tb))
            print >>self.output, exc_hdr.center(self.linewidth, '-')
            print >>self.output, tb.rstrip()
        print >>self.output, ('-' * self.linewidth) + "\n"

        # Flush the output
        self.output.flush()

    def resources(self, res_msgs):
        """
        Called by main() if errors were encountered while tearing down
        test resources.  The ``res_msgs`` argument is a list of tuples
        containing two elements: the first element is the Resource
        instance affected; and the second element is a three-element
        tuple returned by sys.exc_info().
        """

        # Emit resource error data
        print >>self.output, ("The following resource teardown errors were "
                              "encountered:")
        for res, (exc_type, exc_value, tb) in res_msgs:
            exc_hdr = ' Resource %s ' % res.__class__.__name__
            if len(res.key) > 1:
                exc_hdr += '(%s) ' % ', '.join(res.key[1:])
            tb = ''.join(traceback.format_exception(exc_type, exc_value, tb))
            print >>self.output, exc_hdr.center(self.linewidth, '-')
            print >>self.output, tb.rstrip()

        # Flush the output
        self.output.flush()

    def info(self, message):
        """
        Called to emit other specialized messages not specifically
        categorized.  Currently only used in the case of dependency
        cycle detection.  The ``message`` argument will be an
        explanatory message.
        """

        # Emit the message
        print >>self.output, '\n' + message

        # Flush the output
        self.output.flush()

    def status(self, dt, message):
        """
        Called to emit status messages printed to the dtest.result
        stream.  The ``dt`` argument will be the test descriptor, and
        the ``message`` argument will be a message string or, if using
        the ``print`` statement, bare whitespace.  The default
        implementation ignores messages consisting of whitespace and
        writes non-whitespace messages, prefixed with the short name
        of ``dt``, to the ``self.output`` stream; the output stream
        will be flushed to ensure the message is emitted.
        """

        # Ignore messages composed only of whitespace...
        if message.isspace():
            return

        # Get the short name of the test...
        shname = str(dt)
        if '.' in shname:
            dummy, shname = shname.rsplit('.', 1)

        # Emit the message
        print >>self.output, "%s: %s" % (shname, message)

        # Flush the output
        self.output.flush()


class DTestQueue(object):
    """
    DTestQueue
    ==========

    The DTestQueue class maintains a queue of tests waiting to be run.
    The constructor initializes the queue to an empty state and stores
    a maximum simultaneous thread count ``maxth`` (None means
    unlimited); a ``skip`` evaluation routine (defaults to testing the
    ``skip`` attribute of the test); and an instance of DTestOutput.
    The list of all tests in the queue is maintained in the ``tests``
    attribute; tests may be added to a queue with add_test() (for a
    single test) or add_tests() (for a sequence of tests).  The tests
    in the queue may be run by invoking the run() method.
    """

    def __init__(self, maxth=None, skip=lambda dt: dt.skip,
                 output=DTestOutput()):
        """
        Initialize a DTestQueue.  The ``maxth`` argument must be
        either None or an integer specifying the maximum number of
        simultaneous threads permitted.  The ``skip`` arguments is
        function references; it should take a test and return True if
        the test should be skipped.  The ``output`` argument should be
        an instance of DTestOutput containing a notify() method, which
        takes a test and the state to which it is transitioning, and
        may use that information to emit a test result.  Note that the
        notify() method will receive state transitions to the RUNNING
        state, as well as state transitions for test fixtures; callers
        may find the DTestBase.istest() method useful for
        differentiating between regular tests and test fixtures for
        reporting purposes.
        """

        # Save our maximum thread count
        if maxth is None:
            self.sem = None
        else:
            self.sem = Semaphore(maxth)

        # Need to remember the skip routine
        self.skip = skip

        # Also remember the output
        self.output = output

        # Initialize the lists of tests
        self.tests = set()
        self.waiting = None
        self.runlist = set()

        # No initial resource manager...
        self.res_mgr = resource.ResourceManager()

        # Need locks for the waiting and runlist lists
        self.waitlock = Semaphore()
        self.runlock = Semaphore()

        # Set up some statistics...
        self.th_count = 0
        self.th_event = Event()
        self.th_simul = 0
        self.th_max = 0

        # Place to keep any exceptions we encounter within dtest
        # itself
        self.caught = []

        # We're not yet running
        self.running = False

    def add_test(self, tst):
        """
        Add a test ``tst`` to the queue.  Tests can be added multiple
        times, but the test will only be run once.
        """

        # Can't add a test if the queue is running
        if self.running:
            raise DTestException("Cannot add tests to a running queue.")

        # First we need to get the test object
        dt = test._gettest(tst)

        # Add it to the set of tests
        self.tests.add(dt)

    def add_tests(self, tests):
        """
        Add a sequence of tests ``tests`` to the queue.  Tests can be
        added multiple times, but the test will only be run once.
        """

        # Can't add a test if the queue is running
        if self.running:
            raise DTestException("Cannot add tests to a running queue.")

        # Run add_test() in a loop
        for tst in tests:
            self.add_test(tst)

    def dot(self, grname='testdeps'):
        """
        Constructs a GraphViz-compatible dependency graph with the
        given name (``testdeps``, by default).  Returns the graph as a
        string.  The graph can be fed to the ``dot`` tool to generate
        a visualization of the dependency graph.  Note that red nodes
        in the graph indicate test fixtures, and red dashed edges
        indicate dependencies associated with test fixtures.  If the
        node outline is dotted, that indicates that the test was
        skipped in the most recent test run.
        """

        # Helper to generate node and edge options
        def mkopts(opts):
            # If there are no options, return an empty string
            if not opts:
                return ''

            # OK, let's do this...
            return ' [' + ','.join(['%s="%s"' % (k, opts[k])
                                    for k in opts]) + ']'

        # Now, create the graph
        nodes = []
        edges = []
        for dt in sorted(self.tests, key=lambda dt: str(dt)):
            # Get the real test function
            tfunc = dt.test

            # Make the node
            opts = dict(label=r'%s\n%s:%d' %
                        (dt, tfunc.func_code.co_filename,
                         tfunc.func_code.co_firstlineno))
            if dt.state:
                opts['label'] += r'\n(Result: %s)' % dt.state
            if (dt.state == FAIL or dt.state == XFAIL or dt.state == ERROR or
                dt.state == DEPFAIL):
                opts['color'] = 'red'
            elif isinstance(dt, test.DTestFixture):
                opts['color'] = 'blue'
            if dt.state == SKIPPED:
                opts['style'] = 'dotted'
            elif dt.state == DEPFAIL:
                opts['style'] = 'dashed'
            nodes.append('"%s"%s;' % (dt, mkopts(opts)))

            # Make all the edges
            for dep in sorted(dt.dependencies, key=lambda dt: str(dt)):
                opts = {}
                if (isinstance(dt, test.DTestFixture) or
                    isinstance(dep, test.DTestFixture)):
                    opts.update(dict(color='blue', style='dashed'))
                if dt._partner is not None and dep == dt._partner:
                    opts['style'] = 'dotted'

                edges.append('"%s" -> "%s"%s;' % (dt, dep, mkopts(opts)))

        # Return a graph
        return (('strict digraph "%s" {\n\t' % grname) +
                '\n\t'.join(nodes) + '\n\n\t' + '\n\t'.join(edges) + '\n}')

    def run(self, debug=False):
        """
        Runs all tests that have been queued up.  Does not return
        until all tests have been run.  Causes test results and
        summary data to be emitted using the ``output`` object
        registered when the queue was initialized.
        """

        # Can't run an already running queue
        if self.running:
            raise DTestException("Queue is already running.")

        # OK, put ourselves into the running state
        self.running = True

        # Must begin by ensuring we're monkey-patched
        monkey_patch()

        # OK, let's prepare all the tests...
        for dt in self.tests:
            dt._prepare()

        # Second pass--determine which tests are being skipped
        waiting = []
        for dt in self.tests:
            # Do we skip this one?
            willskip = self.skip(dt)

            # If not, check if it's a fixture with no dependencies...
            if not willskip and not dt.istest():
                if dt._partner is None:
                    if len(dt._revdeps) == 0:
                        willskip = True
                else:
                    if len(dt._revdeps) == 1:
                        willskip = True

            # OK, mark it skipped if we're skipping
            if willskip:
                dt._skipped(self.output)
            else:
                waiting.append(dt)

        # OK, last pass: generate list of waiting tests; have to
        # filter out SKIPPED tests
        self.waiting = set([dt for dt in self.tests if dt.state != SKIPPED])

        # Install the capture proxies...
        if not debug:
            capture.install()

        # Spawn waiting tests
        self._spawn(self.waiting)

        # Wait for all tests to finish
        if self.th_count > 0:
            self.th_event.wait()

        # OK, uninstall the capture proxies
        if not debug:
            capture.uninstall()

        # Now we go through and clean up all left-over resources
        self.res_mgr.release_all()

        # Walk through the tests and output the results
        cnt = {
            OK: 0,
            UOK: 0,
            SKIPPED: 0,
            FAIL: 0,
            XFAIL: 0,
            ERROR: 0,
            DEPFAIL: 0,
            'total': 0,
            'threads': self.th_max,
            }
        for t in self.tests:
            # Get the result object
            r = t.result

            # Update the counts
            cnt[r.state] += int(r.test)
            cnt['total'] += int(r.test)

            # Special case update for unexpected OKs and expected failures
            if r.state == UOK:
                cnt[OK] += int(r.test)
            elif r.state == XFAIL:
                cnt[FAIL] += int(r.test)

            try:
                # Emit the result messages
                self.output.result(r, debug)
            except TypeError:
                # Maybe the output object is written to the older
                # standard?
                self.output.result(r)

        # Emit summary data
        self.output.summary(cnt)

        # If there were resource tearDown exceptions, emit data about
        # them
        msgs = self.res_mgr.messages
        if msgs:
            self.output.resources(msgs)

        # If we saw exceptions, emit data about them
        if self.caught:
            self.output.caught(self.caught)

        # We're done running; re-running should be legal
        self.running = False

        # Return False if there were any unexpected OKs, unexpected
        # failures, errors, or dependency failures
        if (cnt[UOK] > 0 or
            (cnt[FAIL] - cnt[XFAIL]) > 0 or
            cnt[ERROR] > 0 or cnt[DEPFAIL] > 0 or
            len(self.res_mgr.messages) > 0):
            return False

        # All tests passed!
        return True

    def _spawn(self, tests):
        """
        Selects all ready tests from the set or list specified in
        ``tests`` and spawns threads to execute them.  Note that the
        maximum thread count restriction is implemented by having the
        thread wait on the ``sem`` Semaphore after being spawned.
        """

        # Work with a copy of the tests
        tests = list(tests)

        # Loop through the list
        while tests:
            # Pop off a test to consider
            dt = tests.pop(0)

            with self.waitlock:
                # Is test waiting?
                if dt not in self.waiting:
                    continue

                # OK, check dependencies
                elif dt._depcheck(self.output):
                    # No longer waiting
                    self.waiting.remove(dt)

                    # Place test on the run list
                    with self.runlock:
                        self.runlist.add(dt)

                    # Spawn the test
                    self.th_count += 1
                    spawn_n(self._run_test, dt)

                # Dependencies failed; check if state changed and add
                # its dependents if so
                elif dt.state is not None:
                    # No longer waiting
                    self.waiting.remove(dt)

                    # Check all its dependents.  Note--not trying to
                    # remove duplicates, because some formerly
                    # unrunnable tests may now be runnable because of
                    # the state change
                    tests.extend(list(dt.dependents))

    def _run_test(self, dt):
        """
        Execute ``dt``.  This method is meant to be run in a new
        thread.

        Once a test is complete, the thread's dependents will be
        passed back to the spawn() method, in order to pick up and
        execute any tests that are now ready for execution.
        """

        # Acquire the thread semaphore
        if self.sem is not None:
            self.sem.acquire()

        # Increment the simultaneous thread count
        self.th_simul += 1
        if self.th_simul > self.th_max:
            self.th_max = self.th_simul

        # Save the output and test relative to this thread, for the
        # status stream
        status.setup(self.output, dt)

        # Execute the test
        try:
            dt._run(self.output, self.res_mgr)
        except:
            # Add the exception to the caught list
            self.caught.append(sys.exc_info())

            # Manually transition the test to the ERROR state
            dt._result._transition(ERROR, output=self.output)

        # OK, done running the test; take it off the run list
        with self.runlock:
            self.runlist.remove(dt)

        # Now, walk through its dependents and check readiness
        self._spawn(dt.dependents)

        # All right, we're done; release the semaphore
        if self.sem is not None:
            self.sem.release()

        # Decrement the thread count
        self.th_simul -= 1
        self.th_count -= 1

        # If thread count is now 0, signal the event
        with self.waitlock:
            if len(self.waiting) == 0 and self.th_count == 0:
                self.th_event.send()
                return

            # If the run list is empty, that means we have a cycle
            with self.runlock:
                if len(self.runlist) == 0:
                    for dt2 in list(self.waiting):
                        # Manually transition to DEPFAIL
                        dt2._result._transition(DEPFAIL, output=self.output)

                    # Emit an error message to let the user know what
                    # happened
                    self.output.info("A dependency cycle was discovered.  "
                                     "Please examine the dependency graph "
                                     "and correct the cycle.  The --dot "
                                     "option may be useful here.")

                    # Now, let's signal our event
                    self.th_event.send()


def explore(directory=None, queue=None):
    """
    Explore ``directory`` (by default, the current working directory)
    for all modules matching the test regular expression and import
    them.  Each module imported will be further explored for tests.
    This function may be used to discover all registered tests prior
    to running them.  Returns a tuple; the first element is a set of
    all discovered tests, and the second element is a list of tuples
    containing information about all ImportError exceptions caught.
    The elements of this exception information tuple are, in order, a
    path, the module name, and a tuple of exception information as
    returned by sys.exc_info().
    """

    # If no queue is provided, allocate one with the default settings
    if queue is None:
        queue = DTestQueue()

    # Set of all discovered tests
    tests = set()

    # List of all import exceptions
    caught = []

    # Need the allowable suffixes
    suffixes = [sfx[0] for sfx in imp.get_suffixes()]

    # Obtain the canonical directory name
    if directory is None:
        directory = os.getcwd()
    else:
        directory = os.path.abspath(directory)

    # This is the directory we'll be searching
    searchdir = directory

    # But does it have an __init__.py?
    pkgpath = None
    for sfx in suffixes:
        if os.path.exists(os.path.join(directory, '__init__' + sfx)):
            # Refigure the directory
            directory, pkgpath = os.path.split(directory)

    # Now, let's jigger the import path
    tmppath = sys.path
    sys.path = [directory] + sys.path

    # Import the package, if necessary
    if pkgpath is not None:
        try:
            __import__(pkgpath)
            test.visit_mod(sys.modules[pkgpath], tests)
        except ImportError:
            # Remember the exception we got
            caught.append((searchdir, pkgpath, sys.exc_info()))

    # Having done that, we now begin walking the directory tree
    for root, dirs, files in os.walk(searchdir):
        # Let's determine the module's package path
        if root == directory:
            pkgpath = ''
        else:
            sep = root[len(directory)]
            subdir = root[len(directory) + 1:]
            pkgpath = '.'.join(subdir.split(sep)) + '.'

        # Start with files...
        for f in files:
            # Does it match the testRE?
            if not test.testRE.match(f):
                continue

            # Only interested in files we can load
            for sfx in suffixes:
                if f.endswith(sfx):
                    modname = f[:-len(sfx)]
                    break
            else:
                # Can't load it, so skip it
                continue

            # Determine the module's full path
            fullmodname = pkgpath + modname

            # Let's try to import it
            try:
                __import__(fullmodname)
                mod = sys.modules[fullmodname]
            except ImportError:
                # Remember the exception we got
                caught.append((os.path.join(root, f), fullmodname,
                               sys.exc_info()))

                # Can't import it, so move on
                continue

            test.visit_mod(mod, tests)

        # Now we want to determine which subdirectories are packages;
        # they'll contain __init__.py
        subdirs = []
        for d in dirs:
            # Only interested in directories which contain __init__.py
            for sfx in suffixes:
                if os.path.exists(os.path.join(root, d, '__init__' + sfx)):
                    break
            else:
                # Not a package, so skip it
                continue

            # Does it match the testRE?
            if not test.testRE.match(d):
                # No, but let's continue exploring under it
                subdirs.append(d)
                continue

            # Determine the package's full path
            fullpkgname = pkgpath + d

            # Let's try to import it
            try:
                __import__(fullpkgname)
                pkg = sys.modules[fullpkgname]
            except ImportError:
                # Remember the exception we got
                caught.append((os.path.join(root, d), fullpkgname,
                               sys.exc_info()))

                # Can't import it, no point exploring under it
                continue

            test.visit_mod(pkg, tests)

            # We also want to explore under it
            subdirs.append(d)

        # Make sure to set up our pruned subdirectory list
        dirs[:] = subdirs

    # We have finished loading all tests; restore the original import
    # path
    sys.path = tmppath

    # Add the discovered tests to the queue
    queue.add_tests(tests)

    # Output the import errors, if any
    if caught:
        queue.output.imports(caught)

    # Return the queue
    return queue


def main(directory=None, maxth=None, skip=lambda dt: dt.skip,
         output=DTestOutput(), dryrun=False, debug=False, dotpath=None):
    """
    Discover tests under ``directory`` (by default, the current
    directory), then run the tests under control of ``maxth``,
    ``skip``, and ``output`` (see the documentation for the run()
    function for more information on these three parameters).  Returns
    True if all tests (with the exclusion of expected failures)
    passed, or False if an unexpect OK, a failure, or an error was
    encountered.
    """

    # First, allocate a queue
    queue = DTestQueue(maxth, skip, output)

    # Next, discover the tests of interest
    explore(directory, queue)

    # Is this a dry run?
    if not dryrun:
        # Nope, execute the tests
        result = queue.run(debug=debug)
    else:
        result = True

        # Print out the names of the tests
        print "Discovered tests:\n"
        for dt in queue.tests:
            if dt.istest():
                print str(dt)

    # Are we to dump the dependency graph?
    if dotpath is not None:
        with open(dotpath, 'w') as f:
            print >>f, queue.dot()

    # Now, let's return the result of the test run
    return result


def optparser(*args, **kwargs):
    """
    Builds and returns an option parser with the default options
    recognized by the dtest framework.  All arguments are passed to
    the OptionParser constructor.
    """

    # Set up an OptionParser
    op = OptionParser(*args, **kwargs)

    # Set up our default options
    op.add_option("-d", "--directory",
                  action="store", type="string", dest="directory",
                  help="The directory to search for tests to run.")
    op.add_option("-m", "--max-threads",
                  action="store", type="int", dest="maxth",
                  help="The maximum number of tests to run simultaneously; if "
                  "not specified, an unlimited number of tests may run "
                  "simultaneously.")
    op.add_option("-s", "--skip",
                  action="store", type="string", dest="skip",
                  help="Specifies a rule to control which tests are skipped.  "
                  "If value contains '=', tests having an attribute with the "
                  "given value will be skipped.  If value does not contain "
                  "'=', tests that have the attribute will be skipped.")
    op.add_option("--no-skip",
                  action="store_true", dest="noskip",
                  help="Specifies that no test should be skipped.  Overrides "
                  "--skip, if specified.")
    op.add_option("-n", "--dry-run",
                  action="store_true", dest="dryrun",
                  help="Performs a dry run.  After discovering all tests, "
                  "the list of tests is printed to standard output.")
    op.add_option("-D", "--debug",
                  action="store_true", dest="debug",
                  help="Enables debugging mode.  Disables output capturing "
                  "for running tests, causing all output to be emitted "
                  "immediately.")
    op.add_option("--dot",
                  action="store", type="string", dest="dotpath",
                  help="After running tests, a text representation of the "
                  "dependency graph is placed in the indicated file.  This "
                  "file may then be passed to the \"dot\" tool of the "
                  "GraphViz package to visualize the dependency graph.  "
                  "This option may be used in combination with \"-n\".")

    # Return the OptionParser
    return op


def opts_to_args(options):
    """
    Converts an options object--as returned by calling the
    parse_args() method of the return value from the optparser()
    function--into a dictionary that can be fed to the main() function
    to execute the desired test operation.
    """

    # Build the arguments dictionary
    args = {}

    # Start with the skip-related arguments
    if options.noskip is True:
        args['skip'] = lambda dt: False
    elif options.skip is not None:
        if '=' in options.skip:
            k, v = options.skip.split('=', 1)
            args['skip'] = lambda dt: getattr(dt, k, None) == v
        else:
            args['skip'] = lambda dt: hasattr(dt, options.skip)

    # Now look at max threads
    if options.maxth is not None:
        args['maxth'] = options.maxth

    # Are we doing a dry run?
    if options.dryrun is True:
        args['dryrun'] = True

    # Are we in debug mode?
    if options.debug is True:
        args['debug'] = True

    # How about dumping the dependency graph?
    if options.dotpath is not None:
        args['dotpath'] = options.dotpath

    # And, finally, directory
    if options.directory is not None:
        args['directory'] = options.directory

    # Return the built arguments object
    return args


if __name__ == '__main__':
    # Obtain the options
    opts = optparser(usage="%prog [options]")

    # Process command-line arguments
    (options, args) = opts.parse_args()

    # Execute the test suite
    sys.exit(not main(**opts_to_args(options)))



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
==============
Test Constants
==============

This module contains the various constants used by the test framework.
The constants are the various states that a test may be in (RUNNING,
FAIL, XFAIL, ERROR, DEPFAIL, OK, UOK, and SKIPPED) and the origins of
messages in the result (PRE, POST, and TEST).
"""

# Test states
RUNNING = 'RUNNING'  # test running
FAIL = 'FAIL'        # test failed
XFAIL = 'XFAIL'      # test expected to fail
ERROR = 'ERROR'      # error running test
DEPFAIL = 'DEPFAIL'  # dependency failed or errored out
OK = 'OK'            # test completed successfully
UOK = 'UOK'          # test unexpectedly completed successfully
SKIPPED = 'SKIPPED'  # test was skipped


# Result message origins
PRE = 'PRE'    # Error in pre-execute fixture
POST = 'POST'  # Error in post-execute fixture
TEST = 'TEST'  # Error from the test itself
